# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
affects.py

Items which give bonuses to stats or skills.
'''

import olc, mudsys, storage, utils, hooks, auxiliary, random

################################################################################
# item type definition
################################################################################

affect_types = {}

class AffectData:
    '''Data for affects for items.'''

    __item_type__ = 'affect'

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.type = 'cechy'
        self.name = 'Siła'
        self.value = 1

        if set != None:
            self.type = set.readString('type')
            self.name = set.readString('name')
            self.value = set.readInt('value')

    def copyTo(self, to):
        to.type = self.type
        to.name = self.name
        to.value = self.value

    def copy(self):
        newData = AffectData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('type', self.type)
        set.storeString('name', self.name)
        set.storeInt('value', self.value)
        return set

class AffectsData:
    '''Data for affects for chars.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.names = []
        self.types = []
        self.sources = []

        if set != None and len(set.readString('names')) > 0:
            self.names = set.readString('names').split(';')
            self.types = set.readString('types').split(';')
            self.sources = set.readString('sources').split(';')

    def copyTo(self, to):
        to.names = self.names
        to.types = self.types
        to.sources = self.sources

    def copy(self):
        newData = AffectsData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('names', ';'.join(self.names))
        set.storeString('types', ';'.join(self.types))
        set.storeString('sources', ';'.join(self.sources))
        return set

################################################################################
# functions
################################################################################
def generate_affect(self, afftype, maxval = 5, onlyname = False, prefix = [], suffix = []):
    '''Generate random affect on object. If onlyname is set on True, generate only
    name of stat/skill on which item affect. Prefix and suffix must have that same
    amount of elements and in this same order like affect type. At this moment, for
    stats is 4.'''
    if afftype == 'cechy':
        if len(prefix) > 4 or len(suffix) > 4:
            return
        names = affect_types['cechy']
    else:
        if len(prefix) > len(afftype) or len(suffix) > len(afftype):
            return
        names = afftype
    if onlyname:
        val = maxval
    else:
        if maxval > 0:
            val = random.randint(1, maxval)
        else:
            val = random.randint(maxval, -1)
    index = random.randint(0, len(names) - 1)
    self.get_type_data('affect').name = names[index]
    self.get_type_data('affect').value = val
    if len(prefix) > 0:
        self.name = prefix[index] + ' ' + self.name
    elif len(suffix) > 0:
        self.name += ' ' + suffix[index]

################################################################################
# Affects OLC
################################################################################
__AFFECTOLC_TYPE__  = 1
__AFFECTOLC_NAME__  = 2
__AFFECTOLC_VALUE__ = 3

affectolc_opt_map = {
    '1' : (__AFFECTOLC_TYPE__, 'Wybierz typ: '),
    '2' : (__AFFECTOLC_NAME__, 'Wprowadź nazwę cechy/umiejętności: '),
    '3' : (__AFFECTOLC_VALUE__, 'Wprowadź wysokość premii: ')
    }

def affectolc_menu(sock, data):
    '''display our menu interface.'''
    buf = []
    buf.append('{g1)Typ               : {c' + data.type)
    buf.append('{g2)Cecha/Umiejętność : {c' + data.name)
    buf.append('{g3)Premia            : {c' + str(data.value))
    sock.send('\r\n'.join(buf))

def affectolc_chooser(sock, data, option):
    '''figure out what we want to edit and display a prompt.'''
    if not affectolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = affectolc_opt_map[option]

    #list available type options
    if ret == __AFFECTOLC_TYPE__:
        utils.olc_display_table(sock, affect_types.keys(), 4)

    #list available stats options
    if data.type == 'cechy' and ret == __AFFECTOLC_NAME__:
        utils.olc_display_table(sock, affect_types['cechy'], 4)

    sock.send_raw(mssg)
    return ret

def affectolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to.'''
    if choice != __AFFECTOLC_NAME__ and data.type != 'umiejętności':
        try:
            arg = int(arg)
        except:
            return False
    
    if choice == __AFFECTOLC_TYPE__:
        opts = affect_types.keys()
        arg = int(arg)
        if arg < 0 or arg >= len(opts):
            return False
        newtype = opts[arg]
        if newtype != data.type:
            data.type = newtype
            data.name = ''
            data.value = 0
    elif choice == __AFFECTOLC_NAME__:
        if data.type == 'cechy':
            opts = affect_types['cechy']
            arg = int(arg)
            if arg < 0 or arg >= len(opts):
                return False
            data.name = opts[arg]
        else:
            data.name = arg
    elif choice == __AFFECTOLC_VALUE__:
        data.value = int(arg)
    else:
        return False
    return True

def affectolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    buf = 'me.get_type_data("affect").type = "' + data.type + '"\n'
    buf += 'me.get_type_data("affect").name = "' + data.name + '"\n'
    buf += 'me.get_type_data("affect").value = ' + str(data.value) + '\n'
    return buf

################################################################################
# game hooks
################################################################################
def affect_equip_hook(info):
    '''add bonus to stats/skills when we wearing new equipment.'''
    ch, obj = hooks.parse_info(info)

    #no point in continuing
    if not obj.istype("affect"):
        return

    afftype = ''
    value = obj.get_type_data('affect').value
    if obj.get_type_data('affect').type == 'cechy':
        index = affect_types['cechy'].index(obj.get_type_data('affect').name)
        ch.aux('stats').values[index] += value
        if index == 0:
            ch.aux('stats').secondvalues[4] += value
            ch.aux('stats').secondvalues[5] += value
        elif index == 3:
            ch.aux('stats').secondvalues[4] += value
            ch.aux('stats').secondvalues[5] += value
            ch.aux('stats').secondvalues[6] += value
            ch.aux('stats').secondvalues[7] += value
        afftype = 'cecha '
    elif obj.get_type_data('affect').type == 'umiejętności':
        skillname = obj.get_type_data('affect').name
        if skillname in ch.aux('skills').names:
            index = ch.aux('skills').names.index(skillname)
            ch.aux('skills').levels[index] += value
        else:
            ch.addskill(skillname, 0, value)
        afftype = 'umiejętność '
    if value > 0:
        affprefix = 'Zwiększona '
    else:
        affprefix = 'Zmniejszona '
    ch.aux('affects').names.append(affprefix + afftype + obj.get_type_data('affect').name)
    ch.aux('affects').types.append('stałe')
    ch.aux('affects').sources.append(obj.name)

def affect_unequip_hook(info):
    '''remove bonus to stats/skills when we remove equipment.'''
    ch, obj = hooks.parse_info(info)

    #no point in continuing
    if not obj.istype('affect'):
        return

    value = obj.get_type_data('affect').value
    if obj.get_type_data('affect').type == 'cechy':
        index = affect_types['cechy'].index(obj.get_type_data('affect').name)
        ch.aux('stats').values[index] -= value
        if index == 0:
            ch.aux('stats').secondvalues[4] -= value
            ch.aux('stats').secondvalues[5] -= value
        elif index == 3:
            ch.aux('stats').secondvalues[4] -= value
            ch.aux('stats').secondvalues[5] -= value
            ch.aux('stats').secondvalues[6] -= value
            ch.aux('stats').secondvalues[7] -= value
    elif obj.get_type_data('affect').type == 'umiejętności':
        index = ch.aux('skills').names.index(obj.get_type_data('affect').name)
        ch.aux('skills').levels[index] -= value
    if obj.name in ch.aux('affects').sources:
        index = ch.aux('affects').sources.index(obj.name)
        del ch.aux('affects').names[index]
        del ch.aux('affects').types[index]
        del ch.aux('affects').sources[index]

def affect_desc_info(info):
    '''appends informactions about bonuses from an object to the description
    buffer when person looks at it.'''
    obj, ch = hooks.parse_info(info)

    #no point in continuing
    if not obj.istype('affect'):
        return

    if obj.get_type_data('affect').value > 0:
        bonus = 'podnosi'
    else:
        bonus = 'obniża'
    desc = ' Założony ' + bonus + ' wartość ' + obj.get_type_data('affect').type + ' ' + obj.get_type_data('affect').name +'.'
    ch.look_buf += desc

################################################################################
# game commands
################################################################################
def cmd_affects(ch, cmd, arg):
    '''Usage: wplywy

    Show informations about current affects on character.'''
    if len(ch.aux('affects').names) == 0:
        ch.send('Nic obecnie na ciebie nie wpływa.')
        return
    buf = ['Wpływy:']
    for i in range(0, len(ch.aux('affects').names)):
        arrsource = ch.aux('affects').sources[i].split(';')
        if len(arrsource) > 1:
            source = 'czar rzucony przez ' + arrsource[1]
        else:
            source = 'przedmiot ' + arrsource[0]
        buf.append(ch.aux('affects').names[i] + ' (działanie: ' + ch.aux('affects').types[i] + ') źródło: ' + source)
    ch.send('\r\n'.join(buf))

################################################################################
# initialization
################################################################################
auxiliary.install('affects', AffectsData, 'character')

#add command
mudsys.add_cmd('wplywy', None, cmd_affects, 'player', False)

#add method to obj
mudsys.add_obj_method('generateaffect', generate_affect)

#initialize our item data, and its OLC
mudsys.item_add_type(AffectData.__item_type__, AffectData)
olc.item_add_olc(AffectData.__item_type__,
                 affectolc_menu, affectolc_chooser, affectolc_parser,
                 None, affectolc_to_proto)

#initialize some default affects types
affect_types['cechy'] = ['Siła', 'Zręczność', 'Inteligencja', 'Kondycja']
affect_types['umiejętności'] = []

#display hooks for affects
hooks.add("equip", affect_equip_hook)
hooks.add("unequip", affect_unequip_hook)
hooks.add("append_obj_desc", affect_desc_info)

def __unload__():
    '''remove all hooks.'''
    hooks.remove('equip', affect_equip_hook)
    hooks.remove('unequip', affect_unequip_hook)
    hooks.remove('append_obj_desc', affect_desc_info)
