# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
afk.py

Provide data and functions for handle afk mode.
'''

import hooks, mudsys, mudsock, event

################################################################################
# functions
################################################################################
def remove_afk(ch):
    '''Remove afk bit from selected character.'''
    if ch.prfs == 'afk':
        ch.prfs = 'none'
    else:
        prfs = ch.prfs.split(', ')
        prfs.remove('afk')
        ch.prfs = ', '.join(prfs)
    ch.delvar('afkmessage')
    ch.send('Nie jesteś już nieobecny.')

def set_afk(ch):
    '''Set afk bit for selected character.'''
    if ch.prfs == 'none':
        ch.prfs = 'afk'
    else:
        ch.prfs += ', afk'
    ch.send('Od teraz jesteś nieobecny.')

################################################################################
# game events
################################################################################
def afk_event(owner, void, info):
    '''event for set afk flag for inactive players.'''
    for sock in mudsock.socket_list():
        if sock.idle_time >= 300:
            if sock.account != None:
                if sock.ch != None:
                    set_afk(sock.ch)
    event.start_event(None, 300, afk_event, None, '')

################################################################################
# game hooks
################################################################################
def afk_command_hook(info):
    '''Remove afk flag from player.'''
    ch, cmd, arg = hooks.parse_info(info)

    #no point in continue
    if not ch.hasPrefs('afk') or ch.is_npc or cmd == 'afk':
        return

    remove_afk(ch)

def afk_from_game(info):
    '''Remove afk flag from player when quitting.'''
    ch, = hooks.parse_info(info)

    #no point in continue
    if not ch.hasPrefs('afk') or ch.is_npc:
        return

    remove_afk(ch)

def setafk_hook(info):
    '''hook for autoset afk mode for characters.'''
    event.start_event(None, 300, afk_event, None, '')

################################################################################
# game commands
################################################################################
def cmd_afk(ch, cmd, arg):
    '''Usage: afk [message]

    Set or unset self from afk mode. Message will be send to players who sent
    you message in afk mode.'''
    if ch.hasPrefs('afk'):
        remove_afk(ch)
    else:
        set_afk(ch)
    if len(arg) > 0:
        ch.setvar('afkmessage', arg)
        ch.send('Ustawiłeś wiadomość dla innych podczas nieobecności na: ' + arg)

################################################################################
# initialization
################################################################################
mudsys.add_cmd('afk', None, cmd_afk, 'player', False)

mudsys.create_bit('char_prfs', 'afk')

hooks.add('char_from_game', afk_from_game)
hooks.add('command', afk_command_hook)
hooks.add('start', setafk_hook)

def __unload__():
    '''removes the hooks for afk mode.'''
    hooks.remove('char_from_game', afk_from_game)
    hooks.remove('command', afk_command_hook)
    hooks.remove('start', setafk_hook)
