# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
crafts.py

Provide data and functions for crafts.
'''

import storage, olc, auxiliary, utils, mudsys, hooks, obj, mud, gainlevel, random

################################################################################
# plans data type definition
################################################################################
__plans_file__ = "../lib/misc/plans"
__plans__ = []
plan_names = []

class Plan:
    '''Implement one plan.'''

    def __init__(self, name, set = None):
        '''Standard class constructor.'''
        self.name = name
        self.proto = 'maczuga1@matherion'
        self.resources = ['drewno:1']
        self.difficult = 10
        self.time = 20
        self.cost = 10
        self.skill = 'Stolarstwo'
        self.learntime = 100
        self.tool = 'młotek'
        if set != None:
            self.name = set.readString('name')
            self.proto = set.readString('proto')
            self.resources = set.readString('resources').split(';')
            self.difficult = set.readInt('difficult')
            self.time = set.readInt('time')
            self.cost = set.readInt('cost')
            self.skill = set.readString('skill')
            self.learntime = set.readInt('learntime')
            self.tool = set.readString('tool')

    def store(self):
        '''return a storage set representation of the plan.'''
        set = storage.StorageSet()
        set.storeString('name', self.name)
        set.storeString('proto', self.proto)
        set.storeString('resources', ';'.join(self.resources))
        set.storeInt('difficult', self.difficult)
        set.storeInt('time', self.time)
        set.storeInt('cost', self.cost)
        set.storeString('skill', self.skill)
        set.storeInt('learntime', self.learntime)
        set.storeString('tool', self.tool)
        return set

class PlansData:
    '''Character plans.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.names = []
        self.skills = []
        self.learned = []

        if set != None and len(set.readString('names')) > 0:
            self.names = set.readString('names').split(';')
            self.skills = set.readString('skills').split(';')
            self.learned = set.readString('learned').split(';')

    def copyTo(self, to):
        to.names = self.names
        to.skills = self.skills
        to.learned = self.learned

    def copy(self):
        newData = PlansData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('names', ';'.join(self.names))
        set.storeString('skills', ';'.join(self.skills))
        set.storeString('learned', ';'.join(self.learned))
        return set

################################################################################
# interaction functions
################################################################################
def save_plans(plan = None):
    '''save all of the plans to file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList('plans', list)
    for val in __plans__:
        one_set = storage.StorageSet()
        one_set.storeSet('plan', val.store())
        list.add(one_set)
    set.write(__plans_file__)
    set.close()

def load_plans():
    '''read all of the plans from file.'''
    set = storage.StorageSet(__plans_file__)
    for plan in set.readList('plans').sets():
        one_plan = plan.readSet('plan')
        __plans__.append(Plan(None, one_plan))
        plan_names.append(one_plan.readString('name'))
    set.close()
    return

def get_plan(name):
    '''function return plan statistics.'''
    index = plan_names.index(name)
    return __plans__[index]

def add_plan(self, name, skill, learned = 'N'):
    '''function add new plan to character.'''
    self.aux('plans').names.append(name)
    self.aux('plans').skills.append(skill)
    self.aux('plans').learned.append(learned)
    if learned == 'N':
        index = plan_names.index(name)
        if self.sex == 'mężczyzna':
            suffix = 'eś'
            suffix2 = 'ąłeś'
        else:
            suffix = 'aś'
            suffix2 = 'ęłaś'
        self.aux('cooldowns').names.append('+Nauczył' + suffix + ' się nowego planu: ' + name)
        self.aux('cooldowns').times.append(__plans__[index].learntime)
        if self.is_pc:
            self.send('Rozpocz' + suffix2 + ' naukę planu: ' + name)

################################################################################
# Plans OLC
################################################################################
PLAN_NEW = 1
PLAN_DELETE = 2

def plansolc_menu(sock, data):
    '''Display all plans and values for they.'''
    buf = []
    if len(data.aux('plans').names) == 0:
        buf.append('Brak planów.')
    else:
        for i in range(0, len(data.aux('plans').names)):
            if data.aux('plans').learned[i] == 'Y':
                learned = 'Tak'
            else:
                learned = 'Nie'
            buf.append("%d)Nazwa: %s" % (i, data.aux('plans').names[i]))
            buf.append("Umiejętność: " + data.aux('plans').skills[i])
            buf.append("Nauczony: " + learned)
    buf.append('')
    buf.append('')
    buf.append('N) Nowy plan')
    if len(data.aux('plans').names) > 0:
        buf.append('D) Skasuj plan')
    sock.send('\r\n'.join(buf))

def plansolc_chooser(sock, data, option):
    '''Try pick plan to set.'''
    options = ['n', 'd']
    option = option.lower()
    if not option in options:
        return olc.MENU_CHOICE_INVALID
    if option == 'n':
        if len(plan_names) == 0:
            sock.send("Nie ma dostępnych planów.")
            return olc.MENU_CHOICE_INVALID
        utils.olc_display_table(sock, plan_names, 4)
        sock.send('Wybierz dostępny plan: ')
        return PLAN_NEW
    else:
        if len(data.aux('plans').names) == 0:
            return olc.MENU_CHOICE_INVALID
        sock.send('Numer planu do skasowania: ')
        return PLAN_DELETE

def plansolc_parser(sock, mobile, choice, arg):
    '''Set new plan or delete it.'''
    try:
        arg = int(arg)
    except:
        return False
    if choice == PLAN_DELETE:
        if arg < 0 or arg >= len(mobile.aux('plans').names):
            return False
        del mobile.aux('plans').names[arg]
        del mobile.aux('plans').skills[arg]
        del mobile.aux('plans').learned[arg]
    elif choice == PLAN_NEW:
        if arg < 0  or arg >= len(plan_names):
            return False
        plan = get_plan(plan_names[arg])
        mobile.addplan(plan.name, plan.skill, 'Y')
    else:
        return False
    return True

def plansolc_to_proto(thing):
    '''Save plans.'''
    names = '['
    length = len(thing.aux('plans').names)
    if length > 0:
        for i in range(0, length):
            names += "'" + thing.aux('plans').names[i] + "'"
            if i < length - 1:
                names += ', '
    names += ']'
    skills = '['
    if length > 0:
        for i in range(0, length):
            skills += "'" + thing.aux('plans').skills[i] + "'"
            if i < length - 1:
                skills += ', '
    skills += ']'
    learned = '['
    if length > 0:
        for i in range(0, length):
            learned += "'" + thing.aux('plans').learned[i] + "'"
            if i < length - 1:
                learned += ', '
    learned += ']'
    buf = 'me.aux("plans").names = %s\n' % names
    buf += 'me.aux("plans").skills = %s\n' % skills
    buf += 'me.aux("plans").learned = %s\n' % learned
    return buf

def plansolc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send("Edytuj plany")

def plansolc_choice_exec(sock, data):
    '''Edit plans.'''
    olc.do_olc(sock, plansolc_menu, plansolc_chooser, plansolc_parser, None, data)
    return olc.MENU_NOCHOICE

__PLAN_PROTO__     = 1
__PLAN_RESOURCES__ = 2
__PLAN_DIFFICULT__ = 3
__PLAN_TIME__      = 4
__PLAN_COST__      = 5
__PLAN_SKILL__     = 6
__PLAN_LEARNTIME__ = 7
__PLAN_TOOL__      = 8

planolc_opt_map = {
    '1' : (__PLAN_PROTO__, 'Podaj przedmiot jaki zostanie wykonany: '),
    '2' : (__PLAN_RESOURCES__, 'Podaj surowce jakie są potrzebne, oddzielone średnikiem: '),
    '3' : (__PLAN_DIFFICULT__, 'Podaj trudność planu: '),
    '4' : (__PLAN_TIME__, 'Podaj czas (w sekundach) jak długo wykonuje się dany przedmiot: '),
    '5' : (__PLAN_COST__, 'Podaj zmęczenie jakie powoduje wykonanie danego przedmiotu: '),
    '6' : (__PLAN_SKILL__, 'Podaj umiejętność powiązaną z tym planem: '),
    '7' : (__PLAN_LEARNTIME__, 'Podaj czas (w sekundach) jak długo uczy się tego planu: '),
    '8' : (__PLAN_TOOL__, 'Podaj narzędzie jakie będzie potrzebne do wykonania tego planu: ')
    }

def pedit_menu(sock, data):
    '''Display main menu for add/modify plan.'''
    buf = ['{gNazwa         : ' + data.name,
           '{g1)Przedmiot   : {c' + data.proto,
           '{g2)Surowce     : {c' + ';'.join(data.resources),
           '{g3)Trudność    : {c' + str(data.difficult),
           '{g4)Czas        : {c' + str(data.time),
           '{g5)Koszt       : {c' + str(data.cost),
           '{g6)Umiejętność : {c' + data.skill,
           '{g7)Czas nauki  : {c' + str(data.learntime),
           '{g8)Narzędzie   : {c' + data.tool]
    sock.send('\r\n'.join(buf))

def pedit_chooser(sock, data, option):
    '''figure out what we want to edit in spell and display a prompt.'''
    if not planolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = planolc_opt_map[option]

    sock.send_raw(mssg)
    return ret

def pedit_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to in plan.'''
    options = [__PLAN_DIFFICULT__, __PLAN_TIME__, __PLAN_COST__, __PLAN_LEARNTIME__]
    if choice in options:
        try:
            arg = int(arg)
        except:
            return False
    if choice == __PLAN_PROTO__:
        data.proto = arg
    elif choice == __PLAN_RESOURCES__:
        data.resources = arg.split(';')
    elif choice == __PLAN_DIFFICULT__:
        data.difficult = int(arg)
    elif choice == __PLAN_TIME__:
        data.time = max(int(arg), 1)
    elif choice == __PLAN_COST__:
        data.cost = max(int(arg), 1)
    elif choice == __PLAN_SKILL__:
        data.skill = arg
    elif choice == __PLAN_LEARNTIME__:
        data.learntime = max(int(arg), 10)
    elif choice == __PLAN_TOOL__:
        data.tool = arg
    else:
        return False
    return True

################################################################################
# game hooks
################################################################################
def plan_learned_hook(info):
    '''Send informations about learned plan.'''
    ch, mssg = hooks.parse_info(info)

    #no point in continue
    if not mssg.startswith('Nauczył'):
        return

    parts = mssg.split()
    planname = ' '.join(parts[4:])
    if not planname in ch.aux('plans').names:
        return
    index = ch.aux('plans').names.index(planname)
    ch.aux('plans').learned[index] = 'Y'
    if ch.is_pc:
        ch.send(mssg)

def item_finished_hook(info):
    '''Check did player made an item.'''
    ch, mssg = hooks.parse_info(info)

    #no point in continue
    if not mssg.startswith('Wykonaj'):
        return

    parts = mssg.split()
    name = ' '.join(parts[1:])
    index = plan_names.index(name)
    chroll = random.randint(0, 99) + ch.aux('stats').values[1]
    hasSkill = -1
    skill = __plans__[index].skill
    chance = __plans__[index].difficult
    if skill in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index(skill)
        chroll += ch.aux('skills').levels[hasSkill]
    if chroll >= chance:
        success = True
        bonus = chance
    else:
        success = False
        bonus = 1
    if hasSkill > -1:
        ch.aux('skills').exp[hasSkill] += bonus
    else:
        ch.addskill(skill, bonus)
    ch.aux('stats').secondvalues[1] += bonus
    gainlevel.check_exp(ch, skill, 'skill')
    gainlevel.check_exp(ch, 'dex', 'stat')
    item = obj.load_obj(__plans__[index].proto, ch)
    if not success:
        ch.send('Nie udało ci się wykonać ' + item.name + '$a.')
        mud.extract(item)
    else:
        ch.send('Udało ci się wykonać ' + item.name + '$a.')
        hooks.run("get", hooks.build_info("ch obj", (ch, item)))

################################################################################
# game commands
################################################################################
def cmd_pedit(ch, cmd, arg):
    '''Usage: pedit <name of plan>

    Add or edit plan.'''
    if len(arg) == 0:
        ch.send('Podaj nazwę planu.')
        return
    if arg in plan_names:
        plan = get_plan(arg)
    else:
        plan = Plan(arg)
        plan_names.append(arg)
        __plans__.append(plan)
    olc.do_olc(ch.sock, pedit_menu, pedit_chooser, pedit_parser, save_plans, plan)

def cmd_pdelete(ch, cmd, arg):
    '''Usage: pdelete <name of plan>

    Delete selected plan.'''
    if len(arg) == 0:
        ch.send('Podaj nazwę planu do skasowania.')
        return
    if arg in plan_names:
        index = plan_names.index(arg)
        del plan_names[index]
        del __plans__[index]
        save_plans()
        ch.send("Plan " + arg + " został skasowany.")
    else:
        ch.send("Nie ma takiego planu.")

def cmd_plist(ch, cmd, arg):
    '''Usage: plist

    Show list of plans.'''
    if len(plan_names) == 0:
        ch.send('Brak planów.')
        return
    ch.page('\r\n'.join(plan_names))

def cmd_prename(ch, cmd, arg):
    '''Usage: prename \'<name of plan>\' \'<new name of plan>\'

    Rename selected plan.'''
    if len(arg) == 0:
        ch.send('Prawidłowa składnia to: prename \'<nazwa planu>\' \'<nowa nazwa>\'')
        return
    if arg.count("'") != 4:
        ch.send('Musisz podać starą i nową nazwę planu.')
        return
    index = arg.index("'", 1)
    oldname = arg[1:index]
    index = arg.index("'", index + 1) + 1
    newname = arg[index:-1]
    index = plan_names.index(oldname)
    plan_names[index] = newname
    __plans__[index].name = newname
    save_plans()
    ch.send("Nazwa planu " + oldname + " zmieniona na " + newname + ".")

def cmd_make(ch, cmd, arg):
    '''Usage: wykonaj <object>

    Try to make selected object.'''
    if len(arg) == 0:
        ch.send('Prawidłowa składnia to wykonaj <przedmiot>')
        return
    #did we have know this plan?
    if arg not in ch.aux('plans').names:
        ch.send('Nie wiesz jak wykonać ' + arg + '.')
        return
    index = plan_names.index(arg)
    #check if we have enough materials
    haveResources = True
    for resource in __plans__[index].resources:
        name, amount = resource.split(':')
        res, = mud.parse_args(ch, False, cmd, name, 'obj.world')
        have = obj.count_objs(res.prototypes, ch)
        if int(have) < int(amount):
            haveResources = False
            break
    if not haveResources:
        ch.send('Nie masz materiałów aby zrobić ten przedmiot.')
        return
    #did we have anough stamina for work?
    cost = ch.aux('stats').values[5] + __plans__[index].cost
    if cost > ch.aux('stats').secondvalues[5] and ch.is_pc:
        ch.send('Nie masz sił aby wykonać ten przedmiot.')
        return
    #did we have proper tool for work?
    tool = None
    if __plans__[index].tool != 'none':
        haveTool = False
        for item in ch.inv + ch.eq:
            if __plans__[index].tool in item.prototypes.split(', '):
                haveTool = True
                break
        if not haveTool:
            ch.send('Nie masz odpowiednich narzędzi aby wykonać ten przedmiot.')
            return
        for item in ch.eq:
            if __plans__[index].tool in item.prototypes.split(', '):
                tool = item
                break
        if tool == None:
            ch.send('Nie masz narzędzi w ręce aby wykonać ten przedmiot.')
            return
    if ch.is_pc:
        ch.aux('stats').values[5] = cost
    #take all needed materials
    for resource in __plans__[index].resources:
        name, amount = resource.split(':')
        for i in range(0, int(amount)):
            res, = mud.parse_args(ch, False, cmd, name, 'obj.inv.eq')
            mud.extract(res)
    #start making a item
    ch.aux('cooldowns').names.append('+Wykonaj ' + arg)
    ch.aux('cooldowns').times.append(__plans__[index].time)
    if ch.is_pc:
        if ch.sex == 'mężczyzna':
            suffix = 'ąłeś'
        else:
            suffix = 'ęłaś'
        ch.send('Rozpocz' + suffix + ' wykonywanie przedmiotu: ' + arg)
    if tool != None:
        hooks.run('durability', hooks.build_info('ch obj', (ch, tool)))

def cmd_pinfo(ch, cmd, arg):
    '''Usage: pinfo <name>

    Show informations about selected plan.'''
    if len(arg) == 0:
        ch.send('Prawidłowa składnia to: pinfo <nazwa_planu>')
        return
    if not arg in ch.aux('plans').names:
        ch.send('Nie znasz takiego planu.')
        return
    index = plan_names.index(arg)
    tool = obj.load_obj(__plans__[index].tool)
    buf = ['Nazwa: ' + arg,
           'Materiały: ' + ', '.join(__plans__[index].resources),
           'Czas wykonywania: ' + str(__plans__[index].time),
           'Umiejętność: ' + __plans__[index].skill,
           'Narzędzie: ' + tool.name]
    ch.send('\r\n'.join(buf))

################################################################################
# initialization
################################################################################
load_plans()

auxiliary.install('plans', PlansData, 'character')

mudsys.add_char_method('addplan', add_plan)

#add commands
mudsys.add_cmd('pedit', None, cmd_pedit, 'admin', False)
mudsys.add_cmd('pdelete', None, cmd_pdelete, 'admin', False)
mudsys.add_cmd('plist', None, cmd_plist, 'admin', False)
mudsys.add_cmd('prename', None, cmd_prename, 'admin', False)
mudsys.add_cmd('wykonaj', None, cmd_make, 'player', False)
mudsys.add_cmd('pinfo', None, cmd_pinfo, 'player', False)

olc.extend("medit", "P", plansolc_extend_menu, plansolc_choice_exec, None, None,
           plansolc_to_proto)
olc.extend("pcedit", "P", plansolc_extend_menu, plansolc_choice_exec, None, None, None)

#hooks for plans
hooks.add('cooldown', plan_learned_hook)
hooks.add('cooldown', item_finished_hook)

def __unload__():
    '''remove all hooks.'''
    hooks.remove('cooldown', plan_learned_hook)
    hooks.remove('cooldown', item_finished_hook)
