# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
dreams.py

Functions and events for character dreams.
'''
import hooks, event, random, storage, mudsys

################################################################################
# dreams data type definition
################################################################################
__dreams_file__ = "../lib/misc/dreams"
__dreams__ = []

class Dream:
    '''Implements one dream.'''
    def __init__(self, type, body, set = None):
        '''Standard class constructor.'''
        self.type = type
        self.body = body
        if set != None:
            self.type = set.readString('type')
            self.body = set.readString('body')

    def store(self):
        '''return a storage set representation of the dream.'''
        set = storage.StorageSet()
        set.storeString('type', self.type)
        set.storeString('body', self.body)
        return set

    # various getters and setters for the Dream class
    def setType(self, type): self.type = type
    def setBody(self, body): self.body = body
    def getType(self): return self.type
    def getBody(self): return self.body
            
################################################################################
# interaction functions
################################################################################
def save_dreams():
    '''save all of the dreams to a file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList("dreams", list)
    for val in __dreams__:
        one_set = storage.StorageSet()
        one_set.storeSet("dream", val.store())
        list.add(one_set)
    set.write(__dreams_file__)
    set.close()

def load_dreams():
    '''read all of the dreams from file.'''
    set = storage.StorageSet(__dreams_file__)
    for dream in set.readList("dreams").sets():
        __dreams__.append(Dream(None, None, dream.readSet("dream")))
    set.close()
    return

################################################################################
# Events
################################################################################
def dream_event(owner, void, info):
    '''Event show dreams to player.'''
    if owner == None:
        return
    if owner.pos != 'sleeping':
        return
    rand = random.randint(0, 99)
    if rand > 80:
        dreams = []
        nightmares = []
        i = 0
        for dream in __dreams__:
            if dream.type == 'sen':
                dreams.append(i)
            else:
                nightmares.append(i)
        rand = random.randint(0, 10)
        if rand < 5:
            if len(dreams) > 0:
                rand = random.randint(0, len(dreams) - 1)
                i = dreams[rand]
                owner.send(__dreams__[i].body)
        else:
            if len(nightmares) > 0:
                rand = random.randint(0, len(nightmares) - 1)
                i = nightmares[rand]
                owner.send(__dreams__[i].body)
    event.start_event(owner, 20, dream_event, None, '')

################################################################################
# game commands
################################################################################
def cmd_dreams(ch, cmd, arg):
    '''Usage: sny <dodaj | usun | pokaz> <sen | koszmar> [number]

    Add, delete or show list of dreams in selected kategory.'''
    if len(arg) == 0:
        ch.send("Prawidłowa składnia to: sny <dodaj | usun | pokaz> <sen | koszmar> [numer]")
        return
    options = arg.split()
    if len(options) < 2:
        ch.send("Podaj typ snu.")
        return
    if options[1] != 'sen' and options[1] != 'koszmar':
        ch.send("Prawidłowy typ snu to sen lub koszmar.")
        return
    #show list of dreams
    if options[0] == 'pokaz':
        if len(__dreams__) == 0:
            ch.send("Nie ma jeszcze dodanych snów.")
            return
        if len(options) < 3:
            i = 1
            buf = []
            for dream in __dreams__:
                if dream.type == options[1]:
                    buf.append(str(i) + ": " + dream.body[0:75] + '...')
                    i += 1
            ch.send("\r\n".join(buf))
        else:
            try:
                number = int(options[2])
            except:
                ch.send("To nie jest liczba.")
                return
            i = 1
            dream = ''
            for dr in __dreams__:
                if dr.type == options[1]:
                    if i == number:
                        dream = dr.body
                        break
                    i = i + 1
            if len(dream) == 0:
                ch.send("Nie ma snu o tym numerze.")
                return
            ch.send("Typ: " + options[1])
            ch.send(dream)
    #add a dream
    elif options[0] == 'dodaj':
        if ch.notepad == "":
            ch.send("Twój notatnik jest pusty. Spróbuj coś napisać.")
            return
        __dreams__.append(Dream(options[1], ch.notepad))
        save_dreams()
        ch.send("Sen został dodany.")
    #delete a dream
    elif options[0] == 'usun':
        if len(options) < 3:
            ch.send("Podaj numer snu do usunięcia.")
            return
        try:
            number = int(options[2])
        except:
            ch.send("To nie jest liczba.")
            return
        number -= 1
        if number < 0 or number > len(__dreams__) - 1:
            ch.send("Nie ma takiego snu.")
            return
        i = -1
        for dream in __dreams__:
            if dream.type == options[1]:
                i += 1
            if i == number:
                __dreams__.remove(dream)
                break
        save_dreams()
        ch.send("Sen został usunięty.")
        
################################################################################
# hooks
################################################################################
def sleep_hook(info):
    '''Show dreams to player.'''
    ch, = hooks.parse_info(info)
    if ch == None or not ch.is_pc:
        return
    event.start_event(ch, 20, dream_event, None, '')

################################################################################
# initialization and unloading
################################################################################
load_dreams()

mudsys.add_cmd("sny", None, cmd_dreams, "builder", False)

hooks.add("sleep", sleep_hook)

def __unload__():
    '''things that need to be detached when the module is un/reloaded'''
    hooks.remove("sleep", sleep_hook)
