# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria MUD.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
fight.py

Provide functions for fight PvE and PvP
'''

import mudsys, mud, char, mudsock, utils, random, gainlevel, event, hooks, obj, auxiliary, spells
import damage as damfuncs

################################################################################
# fight type definition
################################################################################

class FightData:
    '''Data for fight.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.ready = 1
        self.gainAtt = False
        self.gainDef = False
        self.damage = 1
        self.damageType = ''
        self.preparation = 1
        self.hitChance = 0
        self.evadeChance = 0
        self.defSkillName = ''
        self.attSkillName = ''
        self.hasAttSkill = False
        self.hasDefSkill = False
        self.armorHit = {}
        self.body = {}
        self.armor = {}
        self.weaponType = ''
        self.attackType = ''
        self.enemy = ''
        self.oldAttackType = ''

    def copyTo(self, to):
        to.ready = self.ready
        to.gainAtt = self.gainAtt
        to.damage = self.damage
        to.damageType = self.damageType
        to.preparation = self.preparation
        to.hitChance = self.hitChance
        to.evadeChance = self.evadeChance
        to.defSkillName = self.defSkillName
        to.attSkillName = self.attSkillName
        to.hasAttSkill = self.hasAttSkill
        to.hasDefSkill = self.hasDefSkill
        to.armorHit = self.armorHit
        to.body = self.body
        to.armor = self.armor
        to.weaponType = self.weaponType
        to.attackType - self.attackType
        to.enemy = self.enemy
        to.oldAttackType = self.oldAttackType

    def copy(self):
        newData = FightData()
        self.copyTo(newData)
        return newData

    def store(self):
        return

################################################################################
# functions
################################################################################

def throw_attack(attacker, defender):
    '''Manipulate attacker weapon (throw attacks).'''
    #drop our weapon
    chWeapon = attacker.get_equip('dzierżone')
    if chWeapon == None:
        get_info(attacker, defender, 'melee')
    weaponName = chWeapon.name
    chWeapon.room = attacker.room
    hooks.run("durability", hooks.build_info("ch obj", (attacker, chWeapon)))
    #if we have this same weapon in inventory, put it in hand, otherwise fight with bare hands
    try:
        newWeapon, = mud.parse_args(attacker[0], True, '', chWeapon.name, "obj.inv")
    except:
        if attacker.is_pc:
            attacker.send("Skończyła ci się amunicja!")
        if defender.is_pc:
            defender.send(attacker.name + " zaczyna walczyć wręcz.")
        return get_info(attacker, defender, False)
    if attacker.is_pc:
        attacker.send("Wyciągasz kolejny pocisk.")
    if defender.is_pc:
        defender.send(attacker.name + " wyciąga kolejny pocisk.")
    attacker.equip(newWeapon)

def shoot_attack(ch):
    '''Manipulate attacker weapon and ammunition (ranged attacks).'''
    #drop our ammo
    ammo = None
    chWeapon = ch.get_equip('dzierżone')
    for item in ch.eq:
        if item.istype('container'):
            for item2 in item.contents:
                if item2.uid == chWeapon.get_type_data('ranged').loaded:
                    ammo = item
                    break
    if ammo == None:
        for item in ch.inv:
            if item.uid == chWeapon.get_type_data('ranged').loaded:
                ammo = item
                break
    ammo.room = ch.room
    ammo.hidden = 0
    chWeapon.get_type_data('ranged').loaded = 0
    chWeapon.get_type_data('ranged').powder = 0
    #check durability our ammo and weapon
    hooks.run("durability", hooks.build_info("ch obj", (ch, ammo)))
    hooks.run("durability", hooks.build_info("ch obj", (ch, chWeapon)))

def get_party(fighter):
    '''Return list with members of fighter party which doing this same fight.'''
    party = []
    for ch in fighter.room.chars:
        if ch.hasvar('partyleader') and len(ch.aux('fight').enemy) > 0 and ch.name != fighter.name:
            if ch.aux('fight').enemy == fighter.aux('fight').enemy and ch.getvar('partyleader') == fighter.getvar('partyleader'):
                party.append(ch)
    return party

def gain_exp(ch, expgain):
    '''Gain experience by attacker(s).'''
    #if we attack in party, we gain more experience
    if ch.aux('fight').attackType == 'party':
        attackParty = get_party(ch)
        if len(attackParty) > 0:
            expgain += ((expgain / 100) * (len(attackParty) * 10))
        attackParty.append(ch)
    else:
        attackParty = [ch]
    for attacker in attackParty:
        if attacker.aux('fight').gainAtt:
            if attacker.aux('fight').attackType != 'spell':
                attacker.aux('stats').secondvalues[0] += expgain
                gainlevel.check_exp(attacker, "str", "stat")
            else:
                attacker.aux('stats').secondvalues[2] += expgain
                gainlevel.check_exp(attacker, "int", "stat")
            if attacker.aux('fight').hasAttSkill > -1:
                attacker.aux('skills').exp[attacker.aux('fight').hasAttSkill] += expgain
            else:
                attacker.addskill(attacker.aux('fight').attSkillName, expgain)
            gainlevel.check_exp(attacker, attacker.aux('fight').attSkillName, "skill")            
        if attacker.aux('fight').gainDef:
            attacker.aux('stats').secondvalues[1] += expgain
            gainlevel.check_exp(attacker, "dex", "stat")
            if attacker.aux('fight').hasDefSkill > -1:
                attacker.aux('skills').exp[attacker.aux('fight').hasDefSkill] += expgain
            else:
                attacker.addskill(attacker.aux('fight').defSkillName, expgain)
            gainlevel.check_exp(attacker, attacker.aux('fight').defSkillName, "skill")
        attacker.aux('fight').attackType = attacker.aux('fight').oldAttackType

def attack_event(owner, data, attackType):
    '''Function made attack and count damage.'''
    #did player have enough stamina for fight?
    attacker = data[0]
    defender = data[1]
    if attackType not in ['shoot', 'spell']:
        if len(attacker.aux('fight').enemy) == 0 or len(defender.aux('fight').enemy) == 0:
            return
    if attackType == 'party':
        both = [attacker, defender]
    else:
        both = [defender, attacker]
    if attacker.is_pc:
        if attacker.aux('stats').values[5] == attacker.aux('stats').secondvalues[5]:
            attacker.send("Nie masz sił aby atakować!")
            if attackType == 'normal':
                both = [defender, attacker]
                event.start_event(owner, 2, attack_event, both, 'normal')
            return
        attacker.aux('stats').values[5] += 1
    if defender.is_pc:
        if defender.aux('stats').values[5] == defender.aux('stats').secondvalues[5]:
            defender.send("Nie masz sił aby się bronić!")
            defender.aux('fight').evadeChance = 0
        else:
            defender.aux('stats').values[5] += 1
    #prepare weapon
    attacker.aux('fight').ready -= 1
    if attacker.aux('fight').ready > 0:
        if attacker.is_pc:
            attacker.send("Bierzesz zamach bronią.")
        if defender.is_pc:
            defender.send(attacker.name + " bierze zamach bronią.")
        if attackType in ['normal', 'party']:
            event.start_event(owner, 2, attack_event, both, 'normal')
        return
    #did attacker hit defender?
    attack = attacker.aux('fight').hitChance + random.randint(0, 99)
    attacker.aux('fight').ready = attacker.aux('fight').preparation
    defense = defender.aux('fight').evadeChance + random.randint(0, 99)
    if defense > attack:
        if defender.is_pc:
            if defender.aux('fight').evadeChance < attack:
                if defender.aux('fight').defSkillName == 'Uniki':
                    message = 'Uniknąłeś ataku '
                else:
                    message = 'Zablokowałeś tarczą atak '
                defender.send(message + attacker.name + "$g.")
                defender.aux('fight').gainDef = True
            else:
                defender.send(attacker.name +  " chybia.")
        if attacker.is_pc:
            if defender.aux('fight').evadeChance < attack:
                if defender.aux('fight').defSkillName == 'Uniki':
                    message = ' uniknął twojego ataku.'
                else:
                    message = ' zablokował tarczą twój atak.'
                attacker.send(defender.name + message)
            else:
                attacker.send("Chybiasz.")
        if attacker.aux('fight').weaponType == 'throw':
            attacker = throw_attack(attacker, defender)
        elif attacker.aux('fight').weaponType == 'ranged':
            shoot_attack(attacker)
        if attackType == "normal":
            event.start_event(owner, 2, attack_event, both, 'normal')
    else:
        damage, hitpart, hitArmor = damfuncs.count_damage(defender.aux('fight').body, defender.aux('fight').armor, attacker.aux('fight').damage, attacker.aux('fight').damageType, attacker.get_equip('dzierżone'), defender)
        if damage > 0:
            defender.aux('stats').values[4] -= damage
            if defender.aux('stats').values[4] < 0:
                defender.aux('stats').values[4] = 0
        if hitArmor:
            defender.aux('fight').armorHit[hitpart] = True
        if attacker.is_pc:
            attacker.send('Trafiasz %s$g w %s.' % (defender.name, hitpart))
            if defender.aux('stats').values[4] > 0:
                attacker.send(defender.name + utils.char_status(defender, 'health') + '.')
            attacker.aux('fight').gainAtt = True
        if defender.is_pc:
            defender.send('%s trafia ciebie w %s.' % (attacker.name, hitpart))
        if attacker.aux('fight').weaponType in ['ranged', 'none'] and defender.aux('stats').values[4] > 0:
            expgain = max(defender.aux('stats').values) / 10
            if expgain < 1:
                expgain = 1
            gain_exp(attacker, expgain)
        #end of fight?
        if defender.aux('stats').values[4] == 0:
            if attacker.is_pc:
                attacker.send(defender.name + " pada martwy na ziemię!")
                attacker.aux('alignment').value -= defender.aux('alignment').value / 10
                hooks.run("alignment", hooks.build_info("ch", (attacker, )))
                expgain = max(defender.aux('stats').values)
                gain_exp(attacker, expgain)
                #check if attacker weapon was damaged
                if attacker.aux('fight').attSkillName != 'Walka wręcz' and attacker.aux('fight').weaponType == 'melee' and attacker.aux('fight').gainAtt:
                    hooks.run("durability", hooks.build_info("ch obj", (attacker, attacker.get_equip('dzierżone'))))
                #check if attacker armor was damaged
                damfuncs.check_armor(attacker, attacker.aux('fight').armorHit)
            if defender.is_pc:
                #check if defender weapon was damaged
                if defender.aux('fight').attSkillName != 'Walka wręcz' and defender.aux('fight').gainAtt and defender.aux('fight').weaponType == 'melee':
                    hooks.run("durability", hooks.build_info("ch obj", (defender, defender.get_equip('dzierżone'))))
                #check if defender armor was damaged
                damfuncs.check_armor(defender, defender.aux('fight').armorHit)
            if attacker.aux('fight').weaponType == 'throw':
                attacker = throw_attack(attacker, defender)
            elif attacker.aux('fight').weaponType == 'ranged':
                shoot_attack(attacker)
            defender.aux('fight').enemy = ''
            if attackType != 'party':
                attacker.aux('fight').enemy = ''
            else:
                #if we are in party attack, continue until here are any party members
                attackParty = get_party(attacker)
                defendParty = get_party(defender)
                if len(defendParty) == 0:
                    for ch in attackParty:
                        ch.aux('fight').enemy = ''
                else:
                    for ch in attackParty:
                        ch.act('zabij ' + defendParty[0].name)
            damfuncs.create_corpse(defender, attacker, 'char')
        else:
            if attacker.aux('fight').weaponType == 'throw':
                attacker = throw_attack(attacker, defender)
            elif attacker.aux('fight').weaponType == 'ranged':
                shoot_attack(attacker)
            if attackType in ['normal', 'party']:
                event.start_event(owner, 2, attack_event, both, attackType)

def get_info(ch, vict, attackType):
    '''Function return battle informations about selected character.'''
    #if mob have RANDOM flag, get random stats for him
    if ch.hasPrefs('random'):
        for i in range (0, 4):
            value = vict.aux('stats').values[i]
            minvalue = max(1, (value - (max((value / 10), 1))))
            maxvalue = max(1, (value + (max((value / 10), 1))))
            rand = random.randint(minvalue, maxvalue)
            ch.aux('stats').values[i] = rand
        ch.aux('stats').values[4] = ch.aux('stats').values[0] + ch.aux('stats').values[3]
    #set all fight things for player
    chDamage = ch.aux('stats').values[0]
    if ch.race == 'kotołak':
        chDamageType = 'cut'
    else:
        chDamageType = 'crush'
    chPreparation  = 1
    chHitChance = ch.aux('stats').values[0]
    chEvadeChance = ch.aux('stats').values[1]
    #get player weapon info
    chWeapon = ch.get_equip('dzierżone')
    hasAttSkill = -1
    if attackType == 'shoot':
        chAttSkillName = chWeapon.get_type_data('ranged').skill
        ammo = None
        powder = None
        for item in ch.eq:
            if item.istype('container'):
                for item2 in item.contents:
                    if item.uid == chWeapon.get_type_data('ranged').loaded:
                        ammo = item
                        break
                    if chWeapon.get_type_data('ranged').powder > 0:
                        if item.uid == chWeapon.get_type_data('ranged').powder:
                            powder = item
                            break
        if ammo == None or powder == None:
            for item in ch.inv:
                if item.uid == chWeapon.get_type_data('ranged').loaded:
                    ammo = item
                    break
                if chWeapon.get_type_data('ranged').powder > 0:
                    if item.uid == chWeapon.get_type_data('ranged').powder:
                        powder = item
                        break
        if powder != None:
            mud.extract(powder)
        if ammo.istype('poison'):
            chWeapon.settype('poison')
            chWeapon.get_type_data('poison').strength = ammo.get_type_data('poison').strength
            chWeapon.get_type_data('poison').value = ammo.get_type_data('poison').value
            chWeapon.get_type_data('poison').duration = ammo.get_type_data('poison').duration
            chWeapon.get_type_data('poison').type = ammo.get_type_data('poison').type
        chDamage += (chDamage * float(ammo.get_type_data("ammo").damage / 100))
        chDamageType = ammo.get_type_data("ammo").subtype
        chWeaponType = 'ranged'
    elif attackType == 'spell':
        chAttSkillName = 'Rzucanie czarów'
        chWeaponType = 'none'
        spellname = ch.getvar('spell')
        spell = spells.get_spell(spellname)
        index = ch.aux('spells').names.index(spellname)
        spelldamage = max((spell.value * float(ch.aux('spells').levels[index] / 100)), 1)
        chDamage = ch.aux('stats').values[2] + spelldamage
        chDamageType = spell.type.split('.')[1]
    else:
        if chWeapon == None or not chWeapon.istype('weapon'):
            chAttSkillName = "Walka wręcz"
            chWeaponType = 'melee'
        else:
            chAttSkillName = chWeapon.get_type_data("weapon").skill
            chDamage += (chDamage * float(chWeapon.get_type_data("weapon").damage / 100))
            chDamageType = chWeapon.get_type_data("weapon").subtype
            chPreparation = chWeapon.get_type_data("weapon").prepare
            chWeaponType = chWeapon.get_type_data("weapon").type
    if chAttSkillName in ch.aux('skills').names:
        hasAttSkill = ch.aux('skills').names.index(chAttSkillName)
        chHitChance += ch.aux('skills').levels[hasAttSkill]
    #get player shield info
    chShield = ch.get_equip('trzymane')
    hasDefSkill = -1
    if chShield == None or not chShield.istype('armor'):
        chDefSkillName = 'Uniki'
    else:
        chDefSkillName = 'Tarcza'
        chEvadeChance += chShield.get_type_data('armor').resistance
    if chDefSkillName in ch.aux('skills').names:
        hasDefSkill = ch.aux('skills').names.index(chDefSkillName)
        chEvadeChance += ch.aux('skills').levels[hasDefSkill]
    #get player armor info
    ch.aux('fight').armor, ch.aux('fight').armorHit, ch.aux('fight').body = damfuncs.get_armor(ch)
    chArmor, chArmorHit, chBody = damfuncs.get_armor(ch)
    #set all data
    ch.aux('fight').damage = chDamage
    ch.aux('fight').damageType = chDamageType
    ch.aux('fight').preparation = chPreparation
    ch.aux('fight').hitChance = chHitChance
    ch.aux('fight').evadeChance = chEvadeChance
    ch.aux('fight').defSkillName = chDefSkillName
    ch.aux('fight').attSkillName = chAttSkillName
    ch.aux('fight').hasDefSkill = hasDefSkill
    ch.aux('fight').hasAttSkill = hasAttSkill
    ch.aux('fight').weaponType = chWeaponType
    ch.aux('fight').ready = 1
    ch.aux('fight').gainAtt = False
    ch.aux('fight').gainDef = False
        
def do_fight(ch, vict, attackType = 'normal'):
    '''Function start fight with mobs.'''
    #set all fight things for attacker
    get_info(ch, vict, attackType)
    ch.aux('fight').attackType = attackType
    #set all fight things for defender
    get_info(vict, ch, 'melee')
    vict.aux('fight').attackType = attackType
    #do only cast spell for magic attack
    if attackType == 'spell':
        if ch.hidden > 0:
            ch.hidden = 0
        if vict.hidden > 0:
            vict.hidden = 0
        both = [ch, vict]
        attack_event(ch, both, attackType)
        return
    #do only shoot for ranged attack
    if attackType == 'shoot':
        if ch.hidden > 0:
            ch.hidden = ch.hidden / 2
        if vict.hidden > 0:
            vict.hidden = 0
        both = [ch, vict]
        attack_event(ch, both, attackType)
        return
    #if attacker is hidden and defender don't spot him, one free attack
    if not vict.cansee(ch):
        if ch.is_pc:
            ch.send('Atak z zaskoczenia!')
            if 'Ukrywanie się' in ch.aux('skills').names:
                hasSkill = ch.aux('skills').names.index('Ukrywanie się')
                ch.aux('skills').exp[hasSkill] += 1
                gainlevel.check_exp(ch, 'Ukrywanie się', 'skill')
            else:
                ch.addskill('Ukrywanie się', 1)
            ch.aux('stats').secondvalues[1] += 1
            gainlevel.check_exp(ch, "dex", "stat")
        both = [ch, vict]
        attack_event(ch, both, 'sudden')
        if vict == None:
            return
        if vict.aux('stats').values[4] == 0:
            return
    elif ch.hidden > 0:
        if vict.is_pc:
            vict.send(ch.name + " próbował(a) ciebie zaskoczyć!")
            if 'Spostrzegawczość'  in vict.aux('skills').names:
                hasSkill = vict.aux('skills').names.index('Spostrzegawczość')
                vict.aux('skills').exp[hasSkill] += 1
                gainlevel.check_exp(vict, 'Spotrzegawczość', 'skill')
            else:
                vict.addskill('Spostrzegawczość', 1)
            vict.aux('stats').secondvalues[1] += 1
            gainlevel.check_exp(vict, "int", "stat")
    if ch.hidden > 0:
        ch.hidden = 0
    if vict.hidden > 0:
        vict.hidden = 0
    #if someone have throw weapon, attack first this who throw (in normal and party fight)
    if ch.aux('fight').weaponType == 'throw' and vict.aux('fight').weaponType == 'melee':
        attackType = 'throw'
        both = [ch, vict]
        attack_event(ch, both, attackType)
        if vict == None:
            return
        if vict.aux('stats').values[4] == 0:
            return
    if ch.aux('fight').weaponType == 'melee' and vict.aux('fight').weaponType == 'throw':
        attackType = 'throw'
        both = [vict, ch]
        attack_event(ch, both, attackType)
        if ch == None:
            return
        if ch.aux('stats').values[4] == 0:
            return
    #if attacker is better warrior, attack first
    if ch.aux('fight').hitChance > vict.aux('fight').hitChance:
        both = [ch, vict]
    else:
        both = [vict, ch]
    ch.aux('fight').attackType = attackType
    event.start_event(ch, 1, attack_event, both, attackType)

################################################################################
# game commands
################################################################################

def cmd_kill(ch, cmd, arg):
    '''Command start fight.'''
    try:
        vict, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
    except: return

    if len(ch.aux('fight').enemy) > 0:
        if ch.is_pc:
            ch.send('Już z kimś walczysz.')
        return

    if vict.is_pc and not ch.room.hasBit('pkarena') and ch.is_pc:
        ch.send("Nie możesz tutaj walczyć z innym graczem.")
        return

    inparty = False
    if len(vict.aux('fight').enemy) > 0:
        if vict.aux('fight').enemy == ch.name:
            if ch.is_pc:
                ch.send(vict.name + ' już z tobą walczy.')
                return
        try:
            other, = mud.parse_args(ch, False, cmd, vict.aux('fight').enemy, 'ch.room')
            if ch.hasvar('partyleader') and other.hasvar('partyleader'):
                if ch.getvar('partyleader') == other.getvar('partyleader'):
                    inparty = True
        except:
            inparty = False
        if not inparty:
            if ch.is_pc:
                ch.send(vict.name + " już z kimś walczy.")
            return

    if vict.hasPrefs('immortal'):
        if ch.is_pc:
            ch.send("Nie możesz zaatakować tej postaci.")
        return

    ch.aux('fight').enemy = vict.name
    ch.aux('fight').oldAttackType = ''
    if not inparty:
        vict.aux('fight').enemy = ch.name
        attackType = 'normal'
    else:
        attackType = 'party'
    if ch.is_pc:
        ch.send("Atakujesz %s." % (vict.name))
    do_fight(ch, vict, attackType)

def cmd_challenge(ch, cmd, arg):
    '''Command challenge other player to duel.'''
    #challenge other player to duel.
    options = ['akceptuj', 'odrzuc', 'cofnij']
    if not arg in options:
        try:
            vict, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
        except:
            return

        if vict.is_npc:
            ch.send("Możesz wyzywać na pojedynek tylko innych graczy.")
            return
        if vict.hasvar('enemy'):
            ch.send(vict.name + " już z kimś walczy.")
            return
        if vict.hasvar('challenge'):
            if vict.getvar('challenge') != ch.name:
                ch.send(vict.name + " już został przez kogoś wyzwany.")
                return
            else:
                ch.send("Już raz wyzwałeś " + vict.name  + ". Poczekaj chwilę na odpowiedź.")
                return
        ch.setvar('challenge', vict.name)
        vict.setvar('challenge', ch.name)
        ch.send("Wyzwałeś " + vict.name + "$g na pojedynek!")
        vict.send(ch.name + " wyzwał Ciebie na pojedynek!")
    elif arg == 'cofnij':
        if not ch.hasvar('challenge'):
            ch.send("Nikogo jeszcze nie wyzwałeś na pojedynek.")
            return
        try:
            vict, = mud.parse_args(ch, True, cmd, ch.getvar('challenge'), "ch.world.noself")
        except:
            vict = None
        ch.delvar('challenge')
        ch.send("Wycofałeś swoje wyzwanie.")
        if vict != None:
            vict.delvar('challenge')
            vict.send(ch.name + " wycofał swoje wyzwanie.")
    elif arg == 'odrzuc':
        if not ch.hasvar('challenge'):
            ch.send("Nikt jeszcze nie wyzwał Ciebie na pojedynek.")
            return
        try:
            vict, = mud.parse_args(ch, True, cmd, ch.getvar('challenge'), "ch.world.noself")
        except:
            vict = None
        ch.send("Odrzuciłeś wyzwanie " + ch.getvar('challenge')  + "$g.")
        ch.delvar('challenge')
        if vict != None:
            vict.delvar('challenge')
            vict.send(ch.name + " odrzucił Twoje wyzwanie.")
    elif arg == 'akceptuj':
        try:
            vict, = mud.parse_args(ch, True, cmd, ch.getvar('challenge'), "ch.world.noself")
        except:
            ch.send("Musicie obaj znajdować się w tym samym miejscu aby walczyć ze sobą.")
            return
        ch.delvar('challenge')
        vict.delvar('challenge')
        vict.send(ch.name + " przyjął Twoje wyzwanie. Rozpoczyna się walka!")
        ch.send("Przyjąłeś wyzwanie " + vict.name + "$g. Rozpoczyna się walka!")
        ch.aux('fight').oldAttackType = ''
        ch.aux('fight').attackType = 'normal'
        do_fight(ch, vict)

def cmd_appraise(ch, cmd, arg):
    '''Command appraise stats of selected player/mob.'''
    try:
        vict, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
    except: return

    better = ['Silniejsz', 'Zwinniejsz', 'Inteligentniejsz', 'Wytrzymalsz', 'Żywotniejsz']
    worse = ['Słabsz', 'Mniej zwinn', 'Mniej inteligentn', 'Mniej wytrzymał', 'Mniej żywotn']
    equal = ['siln', 'zwinn', 'inteligentn', 'wytrzymał', 'żywotn']
    buf = ['Przyglądasz się ' + vict.name + '$d przez chwilę.']

    if vict.hasPrefs('random'):
        buf.append('Jednak nie jesteś w stanie ocenić przeciwnika.')
        ch.send('\r\n'.join(buf))
        return

    if vict.sex == "mężczyzna":
        buf.append('Jest on:')
        suffix = 'y'
    else:
        buf.append('Jest ona:')
        suffix = 'a'

    for i in range(0, 5):
        if ch.aux('stats').values[i] > vict.aux('stats').values[i]:
            buf.append(worse[i] + suffix + " od ciebie.")
        elif ch.aux('stats').values[i] == vict.aux('stats').values[i]:
            buf.append('Tak samo ' + equal[i] + suffix + " jak ty.")
        elif ch.aux('stats').values[i] < vict.aux('stats').values[i]:
            buf.append(better[i] + suffix + " niż ty.")
    ch.send('\r\n'.join(buf))
    if vict.is_pc and vict.cansee(ch):
        vict.send('Zauważasz że ' + ch.name + ' przygląda się uważnie tobie.')

def cmd_shoot(ch, cmd, arg):
    '''Usage strzel <cel>

    Shoot from ranged weapon to selected target.'''
    try:
        vict, = mud.parse_args(ch, True, cmd, arg, 'ch.room.noself')
    except:
        return

    if vict.is_pc and not ch.room.hasBit('pkarena') and ch.is_pc:
        ch.send("Nie możesz tutaj walczyć z innym graczem.")
        return

    if len(vict.aux('fight').enemy) > 0:
        inparty = False
        try:
            other, = mud.parse_args(ch, False, cmd, vict.aux('fight').enemy, 'ch.room.noself')
            if ch.hasvar('partyleader') and other.hasvar('partyleader'):
                if ch.getvar('partyleader') == other.getvar('partyleader'):
                    inparty = True
        except:
            inparty = False           
        if not inparty:
            if ch.is_pc:
                ch.send(vict.name + " już z kimś walczy.")
            return

    if vict.hasPrefs('immortal'):
        if ch.is_pc:
            ch.send("Nie możesz zaatakować tej postaci.")
        return

    chWeapon = ch.get_equip('dzierżone')
    if chWeapon == None or not chWeapon.istype('ranged'):
        ch.send('Nie masz broni dystansowej aby strzelać.')
        return
    if chWeapon.get_type_data('ranged').loaded == 0:
        ch.send('Twoja broń dystansowa nie jest załadowana.')
        return
    for cooldown in ch.aux('cooldowns').names:
        if cooldown.startwith('+Załadował'):
            ch.send('Nie możesz strzelać z tej broni, ponieważ dopiero ją ładujesz.')
            return
    ch.send("Strzelasz do " + vict.name + '$g.')
    if vict.is_pc:
        vict.send(ch.name + ' strzela do ciebie!')
    ch.aux('fight').oldAttackType = ch.aux('fight').attackType
    do_fight(ch, vict, 'shoot')
    # start normal fight if victim is npc
    if vict.is_npc:
        try:
            vict, = mud.parse_args(ch, False, cmd, arg, 'ch.room.noself')
        except:
            return
        vict.act('zabij ' + ch.name)

################################################################################
# fight hooks
################################################################################

def end_fight_hook(info):
    '''When player move from room, end fight - unblock player and mobiles.'''
    ch, rm, ex = hooks.parse_info(info)
    if len(ch.aux('fight').enemy) > 0:
        for char in rm.chars:
            if char.aux('fight') == ch.name:
                char.aux('fight').enemy = ''
                party = get_party(ch)
                if len(party) > 0:
                    char.act('zabij ' + party[0])
        ch.aux('fight').enemy = ''
    
################################################################################
# initialization
################################################################################
auxiliary.install('fight', FightData, 'character')

mudsys.add_cmd("zabij", None, cmd_kill, "player", True)
mudsys.add_cmd("ocen", None, cmd_appraise, "player", True)
mudsys.add_cmd("wyzwanie", None, cmd_challenge, "player", True)
mudsys.add_cmd("strzel", None, cmd_shoot, "player", False)

mudsys.create_bit("room_bits", "pkarena")
mudsys.create_bit('char_prfs', 'random')

hooks.add('exit', end_fight_hook)

def __unload__():
    '''removes the hooks for fight'''
    hooks.remove("exit", end_fight_hook)
