# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
follow.py

Follow after selected character
'''

import mudsys, char, mud, hooks

################################################################################
# game commands
################################################################################
def cmd_follow(ch, cmd, arg):
    '''Usage: podazaj <osoba>

    Start following after selected player.'''
    try:
        tgt, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
    except:
        return
    if tgt.is_npc:
        ch.send("To nie jest gracz.")
        return
    ch.setvar("follow", tgt.name)
    ch.send("Zaczynasz podążać za " + tgt.name + "$i.")
    tgt.send(ch.name + " zaczyna podążać za tobą.")

def cmd_unfollow(ch, cmd, arg):
    '''Usage: stoj

    Stop following after selected player.'''
    if not ch.hasvar("follow"):
        ch.send("Nie podążasz za kimkolwiek.")
        return
    name = ch.getvar("follow")
    ch.delvar("follow")
    ch.send("Przestałeś podążać za " + name + "$i.")
    tgt = mudsys.get_player(name)
    tgt.send(ch.name + " przestał podążać za tobą.")

def cmd_dismiss(ch, cmd, arg):
    '''Usage: odpedz <osoba>

    Dismiss selected follower.'''
    try:
        tgt, = mud.parse_args(ch, True, cmd, arg, "ch.room.noself")
    except:
        return
    if not tgt.hasvar("follow"):
        ch.send(tgt.name + " nie podąża za tobą, więc nie możesz go odpędzić.")
    elif tgt.getvar("follow") != ch.name:
        ch.send(tgt.name + " nie podąża za tobą, więc nie możesz go odpędzić.")
    else:
        tgt.delvar("follow")
        ch.send("Odpędziłeś od siebie " + tgt.name + "$a.")
        tgt.send(ch.name + " odpędził ciebie.")

################################################################################
# game hooks
################################################################################
def follow_exit_hook(info):
    '''follow after selected player. If he exit from room, player exit too.'''
    ch, rm, ex = hooks.parse_info(info)
    if ch.is_npc:
        return
    dir_name = ["północ", "wschód", "południe", "zachód", "północny-wschód", "północny-zachód",
                 "południowy-zachód", "południowy-wschód", "góra", "dół"]
    dir_abbr = ["pn", "w", "pd", "z", "pnw", "pnz", "pdz", "pdw", "g", "d"]
    index = dir_name.index(rm.exdir(ex))
    # move followers after character
    for char in rm.chars:
        if char.hasvar("follow"):
            if char.getvar("follow") == ch.name:
                char.act(dir_abbr[index])

################################################################################
# initialization
################################################################################

# add commands
mudsys.add_cmd("podazaj", None, cmd_follow, "player", False)
mudsys.add_cmd("stoj", None, cmd_unfollow, "player", False)
mudsys.add_cmd("odpedz", None, cmd_dismiss, "player", False)

# exit hook for follow
hooks.add("exit", follow_exit_hook)

def __unload__():
    '''remove all hooks.'''
    hooks.remove('exit', follow_exit_hook)
