# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
gathering.py

Provide data and functions for gathering materials.
'''

import storage, olc, utils, mudsys, hooks, obj, random, mud, gainlevel

################################################################################
# item type definition
################################################################################

res_types = {'plant' : 'Zielarstwo', 'mineral' : 'Górnictwo', 'tree' : 'Drwalnictwo'}

class ResourceData:
    '''Data for resources.'''

    __item_type__ = "resource"

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.type = "plant"
        self.amount = 1
        self.maxamount = 10
        self.resource = "liscie1@proto"
        self.tool = 'none'
        self.skill = 'Zielarstwo'
        self.difficult = 10

        if set != None:
            self.type = set.readString('type')
            self.amount = set.readInt('amount')
            self.maxamount = set.readInt('maxamount')
            self.resource = set.readString('resource')
            self.tool = set.readString('tool')
            self.skill = set.readString('skill')
            self.difficult = set.readInt('difficult')

    def copyTo(self, to):
        to.type = self.type
        to.amount = self.amount
        to.maxamount = self.maxamount
        to.resource = self.resource
        to.tool = self.tool
        to.skills = self.skill
        to.difficult = self.difficult

    def copy(self):
        newData = ResourceData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('type', self.type)
        set.storeInt('amount', self.amount)
        set.storeInt('maxamount', self.maxamount)
        set.storeString('resource', self.resource)
        set.storeString('tool', self.tool)
        set.storeString('skill', self.skill)
        set.storeInt('difficult', self.difficult)
        return set

################################################################################
# Resource OLC
################################################################################
__RESOLC_TYPE__      = 1
__RESOLC_AMOUNT__    = 2
__RESOLC_MAXAMOUNT__ = 3
__RESOLC_RESOURCE__  = 4
__RESOLC_TOOL__      = 5
__RESOLC_DIFF__      = 6

resolc_opt_map = {
    '1' : (__RESOLC_TYPE__, "Wybierz typ: "),
    '2' : (__RESOLC_AMOUNT__, "Podaj obecną ilość zasobów: "),
    '3' : (__RESOLC_MAXAMOUNT__, "Podaj maksymalną ilość zasobów: "),
    '4' : (__RESOLC_RESOURCE__, "Podaj nazwę zasobu do zbierania: "),
    '5' : (__RESOLC_TOOL__, "Podaj nazwę narzędzia do zbierania lub none: "),
    '6' : (__RESOLC_DIFF__, "Podaj trudność zbierania zasobów: ")
    }

def resolc_menu(sock, data):
    '''display our menu interface.'''
    buf = []
    buf.append('{g1) Typ                    : {c' + data.type)
    buf.append('{gUmiejętność               : {c' + data.skill)
    buf.append('{g2) Ilość                  : {c' + str(data.amount))
    buf.append('{g3) Maksymalna ilość       : {c' + str(data.maxamount))
    buf.append('{g4) Otrzymywany zasób      : {c' + data.resource)
    buf.append('{g5) Narzędzie do zbierania : {c' + data.tool)
    buf.append('{g6) Trudność zbierania     : {c' + str(data.difficult))
    sock.send('\r\n'.join(buf))

def resolc_chooser(sock, data, option):
    '''figure out what field we want to edit, and display a prompt.'''
    if not resolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = resolc_opt_map[option]

    if ret == __RESOLC_TYPE__:
        utils.olc_display_table(sock, res_types.keys(), 4)
    sock.send_raw(mssg)
    return ret

def resolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to.'''
    if not choice in [__RESOLC_RESOURCE__, __RESOLC_TOOL__]:
        try:
            arg = int(arg)
        except:
            return False

    if choice == __RESOLC_TYPE__:
        arg = int(arg)
        if arg < 0 or arg >= len(res_types.keys()):
            return False
        rtype = res_types.keys()[arg]
        data.type = rtype
        data.skill = res_types[rtype]
    elif choice == __RESOLC_AMOUNT__:
        data.amount = max(int(arg), 0)
    elif choice == __RESOLC_MAXAMOUNT__:
        data.maxamount = max(int(arg), 1)
    elif choice == __RESOLC_RESOURCE__:
        data.resource = arg
    elif choice == __RESOLC_TOOL__:
        data.tool = arg
    elif choice == __RESOLC_DIFF__:
        data.difficult = max(int(arg), 1)
    else:
        return False
    return True

def resolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    buf = 'me.get_type_data("resource").type = "' + data.type + '"\n'
    buf += 'me.get_type_data("resource").amount = ' + str(data.amount) + '\n'
    buf += 'me.get_type_data("resource").maxamount = ' + str(data.maxamount) + '\n'
    buf += 'me.get_type_data("resource").resource = "' + data.resource + '"\n'
    buf += 'me.get_type_data("resource").tool = "' + data.tool + '"\n'
    buf += 'me.get_type_data("resource").skill = "' + data.skill + '"\n'
    buf += 'me.get_type_data("resource").difficult = ' + str(data.difficult) + '\n'
    return buf

################################################################################
# game hooks
################################################################################
def resource_desc_info(info):
    '''append informations about resource available from object to description
    buffer when person looks at it.'''
    item, ch = hooks.parse_info(info)

    #no point in continuing
    if not item.istype('resource'):
        return
    if item.get_type_data('resource').type == 'plant':
        rtype = 'tej rośliny'
    elif item.get_type_data('resource').type == 'mineral':
        rtype = 'tego minerału'
    else:
        rtype = 'tego drzewa'
    desc = ' Z ' + rtype + ' możesz jeszcze ' + str(item.get_type_data('resource').amount) + ' razy zbierać ' + item.get_type_data('resource').resource + '.'
    if item.get_type_data('resource').tool != 'none':
        desc += ' Aby zdobyć te surowce, potrzebujesz narzędzia: ' + item.get_type_data('resource').tool + '.'
    ch.look_buf += desc

def resource_reset_zone(info):
    '''grow a plant resources.'''
    area, = hooks.parse_info(info)
    plants = []
    #get all plants in this zone
    for item in obj.obj_list():
        if item.room == None:
            continue
        if item.room.locale == area and item.istype('resource'):
            if item.get_type_data('resource').type in ['plant', 'tree']:
                plants.append(item)
    #grow plant
    for plant in plants:
        chance = float(1 - float(plant.get_type_data('resource').amount / plant.get_type_data('resource').maxamount)) * 100
        rand = random.randint(0, 99)
        if rand < chance:
            plant.get_type_data('resource').amount += 1

################################################################################
# game commands
################################################################################
def cmd_gather(ch, cmd, arg):
    '''Usage: zbieraj <object>

    Gather resources from selected object.'''
    try:
        resource, = mud.parse_args(ch, True, cmd, arg, 'obj.room')
    except:
        return
    if not resource.istype('resource'):
        ch.send('Tego nie można zbierać.')
        return
    tool = None
    #check if we have proper tool to gather
    if resource.get_type_data('resource').tool != 'none':
        haveTool = False
        for item in ch.inv + ch.eq:
            if item.name == resource.get_type_data('resource').tool:
                haveTool = True
                break
        if not haveTool:
            ch.send('Nie masz odpowiednich narzędzi aby zbierać materiały.')
            return
        for item in ch.eq:
            if item.name == resource.get_type_data('resource').tool:
                tool = item
                break
        if tool == None:
            ch.send('Nie masz narzędzi w ręce aby zbierać materiały.')
            return
    rtype = resource.get_type_data('resource').type
    if rtype == 'plant':
        cost = 10
    else:
        cost = 20
    cost += ch.aux('stats').values[5]
    if cost > ch.aux('stats').secondvalues[5]:
        ch.send('Nie masz sił aby pracować, odpocznij trochę.')
        return
    ch.aux('stats').values[5] = cost
    #count chance to gather materials from object
    skill = resource.get_type_data('resource').skill
    chance = resource.get_type_data('resource').difficult
    if skill == 'Zielarstwo':
        statindex = 1
        stat = 'dex'
    else:
        statindex = 0
        stat = 'str'
    chroll = ch.aux('stats').values[statindex] + random.randint(0, 99)
    hasSkill = -1
    if skill in ch.aux('skills').names:
        hasSkill = ch.aux('skills').names.index(skill)
        chroll += ch.aux('skills').levels[hasSkill]
    if chroll >= chance:
        success = True
        bonus = chance
    else:
        success = False
        bonus = 1
    if hasSkill > -1:
        ch.aux('skills').exp[hasSkill] += bonus
    else:
        ch.addskill(skill, bonus)
    ch.aux('stats').secondvalues[statindex] += bonus
    gainlevel.check_exp(ch, skill, 'skill')
    gainlevel.check_exp(ch, stat, 'stat')
    if tool != None:
        hooks.run('durability', hooks.build_info("ch obj", (ch, tool)))
    if not success:
        ch.send('Nie udało ci się zebrać ' + resource.get_type_data('resource').resource + ' z ' + resource.name + '$g.')
        return
    #check how much items we gather
    rand = random.randint(1, resource.get_type_data('resource').amount)
    for i in range(0, rand):
        item = obj.load_obj(resource.get_type_data('resource').resource, ch)
        hooks.run("get", hooks.build_info("ch obj", (ch, item)))
    if rand == 1:
        name = item.name
    else:
        name = item.mname
    ch.send('Zdobywasz ' +  name + '.')
    #destroy or reduce our resource
    resource.get_type_data('resource').amount -= rand
    if resource.get_type_data('resource').amount == 0:
        if rtype == 'plant':
            mud.message(ch, None, resource, None, False, 'to_char', '$o usycha.')
            mud.message(ch, None, resource, None, False, 'to_room', '$o usycha.')
        elif rtype == 'tree':
            mud.message(ch, None, resource, None, False, 'to_char', 'Resztki $o$$g powoli znikają.')
            mud.message(ch, None, resource, None, False, 'to_room', 'Resztki $o$$g powoli znikają.')
        else:
            mud.message(ch, None, resource, None, False, 'to_char', '$o zmienia się w pył.')
            mud.message(ch, None, resource, None, False, 'to_room', '$o zmienia się w pył.')
        mud.extract(resource)

################################################################################
# initialization
################################################################################

# initialize our item data, and its OLC
mudsys.item_add_type(ResourceData.__item_type__, ResourceData)
olc.item_add_olc(ResourceData.__item_type__,
                 resolc_menu, resolc_chooser, resolc_parser,
                 None, resolc_to_proto)

#display hooks for resources
hooks.add("append_obj_desc", resource_desc_info)
hooks.add("reset_zone", resource_reset_zone)

#add command
mudsys.add_cmd('zbieraj', None, cmd_gather, 'player', False)

def __unload__():
    '''remove all hooks.'''
    hooks.remove("append_obj_desc", resource_desc_info)
    hooks.remove("reset_zone", resource_reset_zone)
