# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
help.py

Provide various functions for help system.
'''

import hooks, storage, mudsys

################################################################################
# no help data type definition
################################################################################
__nohelps_file__ = '../lib/misc/nohelp'
__nohelps__ = {}

class NoHelp:
    '''Implements one no help entry.'''
    def __init__(self, keyword, amount = 1, set = None):
        '''Standard class constructor.'''
        self.keyword = keyword
        self.amount = amount
        if set != None:
            self.keyword = set.readString('keyword')
            self.amount = set.readInt('amount')

    def store(self):
        '''return a storage set representation of the no help entry.'''
        set = storage.StorageSet()
        set.storeString('keyword', self.keyword)
        set.storeInt('amount', self.amount)
        return set

################################################################################
# interaction functions
################################################################################
def save_nohelp():
    '''save all of no help entries to a file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList('nohelps', list)
    for key, value in __nohelps__.items():
        one_set = storage.StorageSet()
        nohelp = NoHelp(key, value)
        one_set.storeSet('nohelp', nohelp.store())
        list.add(one_set)
    set.write(__nohelps_file__)
    set.close()

def load_nohelp():
    '''read all of no help entries from file.'''
    set = storage.StorageSet(__nohelps_file__)
    for nohelp in set.readList('nohelps').sets():
        entry = NoHelp(None, 0, nohelp.readSet('nohelp'))
        __nohelps__[entry.keyword] = entry.amount
    set.close()
    return

################################################################################
# game hooks
################################################################################
def help_hook(info):
    '''Add new entry for not found help file.'''
    keyword, = hooks.parse_info(info)

    if keyword in __nohelps__:
        __nohelps__[keyword] += 1
    else:
        __nohelps__[keyword] = 1
    save_nohelp()

################################################################################
# game commands
################################################################################
def cmd_nohelp(ch, cmd, arg):
    '''Usage: brakpomocy [keyword]

    Without arguments, show list of all no help entries.'''
    if len(arg) == 0:
        buf = []
        if len(__nohelps__.keys()) == 0:
            ch.send('Brak wpisów.')
            return
        for key, val in __nohelps__.items():
            buf.append(key + ' razy: ' + str(val))
        ch.page('\r\n'.join(buf))
    else:
        if not arg in __nohelps__:
            ch.send('Nie ma takiego wpisu.')
            return
        del __nohelps__[arg]
        save_nohelp()
        ch.send('Wpis ' + arg + ' został skasowany.')

################################################################################
# initialization
################################################################################
load_nohelp()

mudsys.add_cmd('brakpomocy', None, cmd_nohelp, 'admin', False)

hooks.add('help_not_found', help_hook)

def __unload__():
    '''removes the hooks for help.'''
    hooks.remove('help_not_found', help_hook)
