# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
hints.py

Provide hints system for MUD.
'''

import storage, mudsys, event, hooks, random, mud

################################################################################
# interaction functions
################################################################################
__hints_file__ = '../lib/misc/hints'
__hints__ = []

def save_hints():
    '''save all of hints to a file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList('hints', list)
    for val in __hints__:
        one_set = storage.StorageSet()
        one_set.storeString('hint', val)
        list.add(one_set)
    set.write(__hints_file__)
    set.close()

def load_hints():
    '''read all of hints from file.'''
    set = storage.StorageSet(__hints_file__)
    for hint in set.readList('hints').sets():
        __hints__.append(hint.readString('hint'))
    set.close()
    return

################################################################################
# game events
################################################################################
def hints_event(owner, void, info):
    '''event for show hints to players.'''
    if len(__hints__) > 0:
        rand = random.randint(0, len(__hints__) - 1)
        mud.message(None, None, None, None, False, 'to_world', __hints__[rand])
    event.start_event(None, 120, hints_event, None, '')

################################################################################
# game hooks
################################################################################
def start_hints_hook(info):
    '''hook start hints system.'''
    event.start_event(None, 120, hints_event, None, '')

################################################################################
# game commands
################################################################################
def cmd_hints(ch, cmd, arg):
    '''Usage: podpowiedzi [dodaj | usun] [numer]

    Add, delete or show hints.'''
    #show list of hints
    if len(arg) == 0:
        if len(__hints__) == 0:
            ch.send('Nie ma jeszcze dodanych podpowiedzi.')
            return
        i = 1
        buf = []
        for hint in __hints__:
            buf.append(str(i) + ": " + hint)
            i += 1
        ch.page('\r\n'.join(buf))
    else:
        args = arg.split()
        #add a hint
        if args[0] == 'dodaj':
            if ch.notepad == '':
                ch.send('Twój notatnik jest pusty. Spróbuj coś napisać.')
                return
            __hints__.append(ch.notepad)
            ch.send('Podpowiedź została dodana.')
        #delete a hint
        if args[0] == 'usun':
            if len(args) < 2:
                ch.send('Podaj numer podpowiedzi do usunięcia.')
            try:
                number = int(args[1]) - 1
            except:
                ch.send('To nie jest liczba.')
                return
            if number < 0 or number > len(__hints__) - 1:
                ch.send('Nie ma takiej podpowiedzi.')
                return
            del __hints__[number]
            ch.send('Podpowiedź została skasowana.')
        save_hints()

################################################################################
# initialization
################################################################################
load_hints()

mudsys.add_cmd('podpowiedzi', None, cmd_hints, 'admin', False)

hooks.add('start', start_hints_hook)

def __unload__():
    '''removes the hooks for hints system.'''
    hooks.remove('start', start_hints_hook)
