# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
languages.py

Provide data and functions for race languages.
'''

import mudsys, mud, auxiliary, storage, olc, mudsock, random, char

################################################################################
# language data type definition
################################################################################
__languages_file__ = "../lib/misc/languages"
__languages__ = []

class Language:
    '''Implements one language.'''
    def __init__(self, name, set = None):
        '''Standard class constructor.'''
        self.name = name
        self.words = []
        if set != None:
            self.name = set.readString('name')
            self.words = set.readString('words').split(';')

    def store(self):
        '''return a storage set representation of the dream.'''
        set = storage.StorageSet()
        set.storeString("name", self.name)
        set.storeString("words", ';'.join(self.words))
        return set

    # various getters and setters for the Language class
    def setName(self, name): self.name = name
    def setWords(self, words): self.words = words
    def getName(self): return self.name
    def getWords(self): return self.words

class LanguageData:
    '''Store informations about know languages by player.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.names = []
        self.values = []
        self.current = 'kupiecki'

        if set != None and len(set.readString("names")) > 0:
            if len(set.readString("current")) > 0:
                self.current = set.readString("current")
            self.names = set.readString("names").split(';')
            if len(self.names) == 0:
                return
            values = set.readString("values").split(';')
            for value in values:
                self.values.append(float(value))

    def copyTo(self, to):
        to.names = self.names
        to.values = self.values
        to.current = self.current

    def copy(self):
        newData = LanguageData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("current", self.current)
        set.storeString("names", ';'.join(self.names))
        values = []
        for value in self.values:
            values.append(str(value))
        set.storeString("values", ';'.join(values))
        return set

################################################################################
# interaction functions
################################################################################
def save_languages():
    '''save all of the languages to a file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList("languages", list)
    for val in __languages__:
        one_set = storage.StorageSet()
        one_set.storeSet("language", val.store())
        list.add(one_set)
    set.write(__languages_file__)
    set.close()

def load_languages():
    '''read all of the languages from file.'''
    set = storage.StorageSet(__languages_file__)
    for language in set.readList("languages").sets():
        __languages__.append(Language(None, language.readSet("language")))
    set.close()
    return

def lang_level(names, values, current):
    '''Count level of knowledge of language.'''
    if not current in names:
        value = 0
    else:
        index = names.index(current)
        value = values[index]
    if value < 25:
        return 0
    elif value > 24 and value < 50:
        return 1
    elif value > 49 and value < 75:
        return 2
    elif value > 74 and value < 100:
        return 3
    else:
        return 4

def lang_send(ch, range, mssg, info, vict = None):
    '''send message to other players (depends on race languages).'''
    #everyone know main language, so simply send message
    if ch.aux('languages').current == 'kupiecki':
        if vict != None:
            mud.message(ch, vict, None, None, False, "to_vict", mssg)
        else:
            mud.message(ch, None, None, None, False, range, mssg)
    else:
        #get lists of players, depends on they knowledge of language.
        knowlang = [[], [], [], [], []]
        lang = ch.aux('languages').current
        #send message to only one player
        if vict != None:
            if vict.hasPrefs('polyglot') or vict.isInGroup('wizard'):
                index = 4
            else:
                index = lang_level(vict.aux('languages').names, vict.aux('languages').values, lang)
            knowlang[index].append(vict)
        #send message to people in this same room
        elif range == 'to_room':
            for char2 in ch.room.chars:
                if char2.hasPrefs('polyglot') or char2.isInGroup('wizard'):
                    index = 4
                else:
                    index = lang_level(char2.aux('languages').names, char2.aux('languages').values, lang)
                if char2 == ch:
                    continue
                knowlang[index].append(char2)
        elif range == 'to_world':
            for char2 in char.char_list():
                if char2.is_pc and char2 != ch:
                    if char2.hasPrefs('polyglot') or char2.isInGroup('wizard'):
                        index = 4
                    else:
                        index = lang_level(char2.aux('languages').names, char2.aux('languages').values, lang)
                    knowlang[index].append(char2)
        #prepare message depends on char knowledge
        words = {"gada: '" : ['gada coś w nieznanym tobie języku.{n', 'gada coś w języku ' + lang + '.{n', 'gada w języku ' + lang + ": '"],
                 "mówi tobie, '" : ['mówi tobie coś w nieznanym tobie języku.{n', 'mówi tobie coś w języku ' + lang + '.{n', 'mówi tobie w języku ' + lang + ", '"],
                 "pyta ciebie, '" : ['pyta ciebie o coś w nieznanym tobie języku.{n', 'pyta ciebie o coś w języku ' + lang + '.{n', 'pyta ciebie w języku ' + lang + ", '"],
                 "mówi: '" : ['mówi coś w nieznanym tobie języku.{n', 'mówi coś w języku ' + lang + '.{n', 'mówi w języku ' + lang + ": '"],
                 "mówi drużynie, '" : ['mówi drużynie coś w nieznanym tobie języku.{n', 'mówi drużynie coś w języku ' + lang + '.{n', 'mówi drużynie w języku ' + lang + ": '"]}
        dwords = {"gada: '" : "gada w nieznanym tobie języku: '",
                  "mówi tobie, '" : "mówi tobie w nieznanym języku, '",
                  "pyta ciebie, '" : "pyta ciebie w nieznanym języku, '",
                  "mówi: '" : "mówi w nieznanym tobie języku: '",
                  "mówi drużynie, '" : "mówi drużynie w nieznanym tobie języku: '"}
        dictionary = []
        for lang2 in __languages__:
            if lang2.name == lang:
                dictionary = lang2.words
                break
        #finally, send message
        tmpmssg = mssg
        for i in [0, 1, 2, 3, 4]:
            if len(knowlang[i]) == 0:
                continue
            mssg = tmpmssg
            for char2 in knowlang[i]:
                if lang in char2.aux('languages').names:
                    index2 = char2.aux('languages').names.index(lang)
                else:
                    index2 = -1
                gain = 0
                if i == 0:
                    index3 = mssg.find(info)
                    if len(dictionary) == 0:
                        mssg = mssg[0:index3] + words[info][0]
                    else:
                        mssg = mssg.replace(info, dwords[info])
                        index3 = mssg.find(dwords[info]) + len(dwords[info])
                        index4 = mssg.rfind("'")
                        words2 = mssg[index3:index4].split()
                        j = 0
                        for word2 in words2:
                            rand = random.randint(0, len(dictionary) - 1)
                            words2[j] = dictionary[rand]
                            j += 1
                        mssg = mssg[0:index3] + ' '.join(words2) + mssg[index4]
                    gain = 0.01
                elif i > 0 and i < 4:
                    rand = random.randint(0, 100) + char2.aux('languages').values[index2]
                    if rand < 100:
                        index3 = mssg.find(info)
                        if len(dictionary) == 0:
                            mssg = mssg[0:index3] + words[info][1]
                        else:
                            mssg = mssg.replace(info, words[info][2])
                            index3 = mssg.find(words[info][2]) + len(words[info][2])
                            index4 = mssg.rfind("'")
                            words2 = mssg[index3:index4].split()
                            j = 0
                            for word2 in words2:
                                rand = random.randint(0, len(dictionary) - 1)
                                words2[j] = dictionary[rand]
                                j += 1
                            mssg = mssg[0:index3] + ' '.join(words2) + mssg[index4]
                        gain = 0.01
                    else:
                        mssg = mssg.replace(info, words[info][2], 1)
                        gain = 0.1
                else:
                    mssg = mssg.replace(info, words[info][2], 1)
                #gain level in language
                if index2 > -1:
                    char2.aux('languages').values[index2] += gain
                    if char2.aux('languages').values[index2] > 100:
                        char2.aux('languages').values[index2] = 100
                else:
                    char2.aux('languages').names.append(lang)
                    char2.aux('languages').values.append(gain)
                #send message
                mud.message(ch, char2, None, None, False, "to_vict", mssg)       
    return mssg

################################################################################
# Languages OLC
################################################################################
LANG_NEW = 1
LANG_EDIT = 2
LANG_DELETE = 3

def langsolc_menu(sock, data):
    '''Display all languages and values for they.'''
    buf = ['Kupiecki: 100%']
    if len(data.aux('languages').names) > 0:
        for i in range(len(data.aux('languages').names)):
            buf.append("%s: %d" % (data.aux('languages').names[i], data.aux('languages').values[i]))
    buf.append('')
    buf.append('')
    buf.append('N) Nowy język')
    if len(data.aux('languages').names) > 0:
        buf.append('E) Edytuj język')
        buf.append('D) Skasuj język')
    sock.send('\r\n'.join(buf))
         
def langsolc_chooser(sock, data, option):
    '''Try pick language to set.'''
    options = ['n', 'e', 'd']
    option = option.lower()
    if not option in options:
        return olc.MENU_CHOICE_INVALID
    if option == 'n':
        sock.send("Nazwa nowego języka: ")
        return LANG_NEW
    else:
        if len(data.aux('languages').names) == 0:
            return olc.MENU_CHOICE_INVALID
        else:
            if option == 'd':
                sock.send("Nazwa języka do skasowania: ")
                return LANG_DELETE
            else:
                sock.send("Nazwa języka do edycji: ")
                return LANG_EDIT

def langsolc_parser(sock, mobile, choice, arg):
    '''Set new language or value for it.'''
    if choice == LANG_EDIT or choice == LANG_DELETE:
        if arg not in mobile.aux('languages').names:
            sock.send('Ta postać nie zna tego języka.')
            return False
        i = mobile.aux('languages').names.index(arg)
    if choice == LANG_DELETE:
        del mobile.aux('languages').names[i]
        del mobile.aux('languages').values[i]
    elif choice == LANG_EDIT:
        olc.do_olc(sock, langolc_menu, langolc_chooser, None, None, [mobile, i])
    elif choice == LANG_NEW:
        mobile.aux('languages').names.append(arg)
        mobile.aux('languages').values.append(0)
        i = mobile.aux('languages').names.index(arg)
        olc.do_olc(sock, langolc_menu, langolc_chooser, None, None, [mobile, i])
    return True

def langsolc_to_proto(thing):
    '''Save languages.'''
    names = '['
    length = len(thing.aux('languages').names)
    if length > 0:
        for i in range(0, length):
            names += "'" + thing.aux('languages').names[i] + "'"
            if i < length - 1:
                names += ', '
    names += ']'
    values = '['
    if length > 0:
        for i in range(0, length):
            values += str(thing.aux('languages').values[i])
            if i < length - 1:
                values += ', '
    values += ']'
    buf = 'me.aux("languages").names = %s\n' % names
    buf += 'me.aux("languages").values = %s\n' % values
    return buf

def langsolc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send("Edytuj języki")

def langsolc_choice_exec(sock, data):
    '''Edit languages.'''
    olc.do_olc(sock, langsolc_menu, langsolc_chooser, langsolc_parser, None, data)

def langolc_menu(sock, data):
    '''Show value for language.'''
    buf = ['Język: ' + data[0].aux('languages').names[data[1]]]
    buf.append('Wartość: ' + data[0].aux('languages').values[data[1]])
    buf.append('Nowa wartość dla języka: ')
    sock.send('\r\n'.join(buf))

def langolc_chooser(sock, data, option):
    '''Set new value for selected language.'''
    try:
        opt = int(option)
        if opt < 0:
            return olc.MENU_CHOICE_INVALID
        data[0].aux('languages').values[data[1]] = opt
        return olc.MENU_NOCHOICE
    except:
        return olc.MENU_CHOICE_INVALID

################################################################################
# game commands
################################################################################
def cmd_languages(ch, cmd, arg):
    '''Usage: admjezyki <dodaj | usun | pokaz> <nazwa> [word]

    Add or delete new language or word to dictionary.'''
    if len(arg) == 0:
        ch.send("Prawidłowa składnia to: admjezyki <dodaj | usun | pokaz> <nazwa> [słowo]")
        return
    options = arg.split()
    if len(options) == 1 and options[0] == 'pokaz':
        buf = ['Znane języki:']
        for lang in __languages__:
            buf.append(lang.name)
        ch.send('\r\n'.join(buf))
        return
    if len(options) < 2:
        ch.send("Podaj nazwę języka.")
        return
    index = -1
    for i in range(0, len(__languages__)):
        if __languages__[i].name == options[1]:
            index = i
            break
    #add, delete or show language
    if len(options) == 2:
        if options[0] == 'pokaz':
            if index == -1:
                ch.send("Nie ma takiego języka.")
                return
            ch.send('Lista słów dla języka : ' + __languages__[index].name)
            if len(__languages__[index].words) == 0:
                ch.send("Brak słów")
            else:
                buf = []
                for word in __languages__[index].words:
                    buf.append(word)
                ch.page('\r\n'.join(buf))
        elif options[0] == 'usun':
            if index == -1:
                ch.send("Nie ma takiego języka.")
                return
            del __languages__[index]
            save_languages()
            ch.send("Język " + options[1] + " został usunięty.")
        elif options[0] == 'dodaj':
            if index != -1:
                ch.send("Już istnieje taki język.")
                return
            __languages__.append(Language(options[1]))
            save_languages()
            ch.send("Język " + options[1] + " został dodany z pustym słownikiem.")
    #add or delete words in selected language
    else:
        if index == -1:
            ch.send("Nie ma takiego jezyka")
            return
        if options[0] == 'dodaj':
            if len(options) != 3:
                ch.send("Podaj słowo do dodania.")
                return
            if options[2] in __languages__[index].words:
                ch.send("Już jest takie słowo w tym języku.")
                return
            __languages__[index].words.append(options[2])
            save_languages()
            ch.send("Dodano do języka " + options[1] + " słowo: " + options[2])
        elif options[0] == 'usun':
            if len(options) != 3:
                ch.send("Podaj słowo do skasowania.")
                return
            __languages__[index].words.remove(options[2])
            save_languages()
            ch.send("Usunięto z języka " + options[1] + " słowo: " + options[2])

def cmd_language(ch, cmd, arg):
    '''Usage: jezyk [zmien] [language]

    Change currently used language.'''
    #show currently used language and know languages
    if len(arg) == 0:
        ch.send("Obecnie mówisz w języku: " + ch.aux('languages').current)
        buf = ['Znane jezyki: ', 'kupiecki: płynnie']
        if len(ch.aux('languages').names) > 0:
            for i in range(0, len(ch.aux('languages').names)):
                value = ch.aux('languages').values[i]
                if value < 25:
                    level = 'bardzo słabo'
                elif value > 24 and value < 50:
                    level = 'słabo'
                elif value > 49 and value < 75:
                    level = 'w miarę'
                elif value > 74 and value < 100:
                    level = 'dobrze'
                else:
                    level = 'płynnie'
                buf.append(ch.aux('languages').names[i] + ": " + level)
        ch.send('\r\n'.join(buf))
        return
    options = arg.split()
    if len(options) != 2:
        ch.send("Prawidłowa składnia to: jezyk zmien nowy_język")
        return
    if options[1] == ch.aux('languages').current:
        ch.send("Posługujesz się już tym językiem.")
        return
    if options[1] != 'kupiecki':
        if options[1] not in ch.aux('languages').names:
            ch.send("Nie znasz tego języka.")
            return
        index = ch.aux('languages').names.index(options[1])
        if ch.aux('languages').values < 50:
            ch.send("Za słabo znasz ten język aby w nim mówić.")
            return
        exist = False
        for lang in __languages__:
            if lang.name == options[1].lower():
                exist = True
                break
        if not exist:
            del ch.aux('languages').names[index]
            del ch.aux('languages').values[index]
            ch.send("Taki język nie istnieje.")
            return
    ch.aux('languages').current = options[1].lower()
    ch.send("Od teraz mówisz w języku " + options[1])

################################################################################
# initialization
################################################################################
load_languages()

auxiliary.install("languages", LanguageData, "character")

mudsys.create_bit('char_prfs', 'polyglot')

#add commands
mudsys.add_cmd("admjezyki", None, cmd_languages, "builder", False)
mudsys.add_cmd("jezyk", None, cmd_language, "player", False)

olc.extend("medit", "L", langsolc_extend_menu, langsolc_choice_exec, None, None,
           langsolc_to_proto)
olc.extend("pcedit", "L", langsolc_extend_menu, langsolc_choice_exec, None, None, None)
