# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
maps.py

Provide functions for show maps of zones and surround.
'''
import mudsys, os, room

################################################################################
# functions
################################################################################
maps_dir = '../lib/maps/'
maps = { }

def load_maps():
    '''Load all maps of zones.'''
    for fmap in os.listdir(maps_dir):
        if os.path.isfile(fmap):
            fin = open(fmap, 'r')
            line = fin.readline()
            zmap = []
            while len(line) > 0:
                zmap.append(line)
            maps[fmap] = '\r\n'.join(zmap)

################################################################################
# game commands
################################################################################
def cmd_map(ch, cmd, arg):
    '''Usage: mapa [obszar]

    Show surrounded map or entire zone.'''
    #show surrounding map
    if len(arg) == 0:
        curroom = []
        for ex in ch.room.exnames:
            if ch.cansee(ch.room.exit(ex)):
                curroom.append(ex)
        if len(curroom) == 0:
            ch.send("Nie ma wyjść z tego pokoju.")
            return
        dir_names = ["północny-zachód", "północ", "północny-wschód", "wschód", "południowy-wschód",
                     "południe", "południowy-zachód", "zachód"]        
        rmap = []
        for i in range (0, 9):
            rmap.append([' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '])
        doors = ['\\', '|', '/', '-', '\\', '|' , '/', '-', 'X', 'o', '{Bo{n']
        indexes = [[-1, -1], [-1, 0], [-1, 1], [0, 1], [1, 1], [1, 0], [1, -1], [0, -1]]
        doors2 = ['\\', '/']
        for mindex in range(0, len(dir_names)):
            if dir_names[mindex] in curroom:
                y = 4 + indexes[mindex][0]
                x = 4 + indexes[mindex][1]
                if rmap[y][x] in doors:
                    if rmap[y][x] in doors2 and rmap[y1][x1] != doors[i]:
                        rmap[y][x] = doors[8]
                else:
                    rmap[y][x] = doors[mindex]
                y += indexes[mindex][0]
                x += indexes[mindex][1]
                rmap[y][x] = doors[9]
                rm = ch.room.exit(dir_names[mindex]).dest
                for i in range(0, len(dir_names)):
                    if rm.exit(dir_names[i]) != None:
                        if ch.cansee(rm.exit(dir_names[i])):
                            y1 = y + indexes[i][0]
                            x1 = x + indexes[i][1]
                            if rmap[y1][x1] in doors:
                                if rmap[y1][x1] in doors2 and rmap[y1][x1] != doors[i]:
                                    rmap[y1][x1] = doors[8]
                            else:
                                rmap[y1][x1] = doors[i]
                            y1 += indexes[i][0]
                            x1 += indexes[i][1]
                            rmap[y1][x1] = doors[9]
        rmap[4][4] = doors[10]
        smap = ['-----------']
        for lmap in rmap:
            smap.append('|' + ''.join(lmap) + '|')
        smap.append('-----------')
        ch.send('\r\n'.join(smap))
    else:
        if ch.room.locale not in maps:
            ch.send("Nie ma mapy dla tego obszaru.")
            return
        ch.page(maps[ch.room.locale])

################################################################################
# initialization
################################################################################
load_maps()

mudsys.add_cmd("mapa", None, cmd_map, "player", False)
