# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
poisons.py

Poison item type.
'''
import olc, mudsys, storage, mud, utils

################################################################################
# item type definition
################################################################################

poison_types = ['damage']

class PoisonData:
    '''Data for poisons.'''

    __item_type__ = "poison"

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.strength = 10
        self.duration = 10
        self.value = 1
        self.type = 'damage'
        if set != None:
            self.strength = set.readInt('strength')
            self.duration = set.readInt('duration')
            self.value  = set.readInt('value')
            self.type = set.readString('type')

    def copyTo(self, to):
        to.strength = self.strength
        to.duration = self.duration
        to.value = self.value
        to.type = self.type

    def copy(self):
        newData = PoisonData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeInt('strength', self.strength)
        set.storeInt('duration', self.duration)
        set.storeInt('value', self.value)
        set.storeString('type', self.type)
        return set

################################################################################
# Poisons OLC
################################################################################
__POIOLC_STRENGTH__ = 1
__POIOLC_DURATION__ = 2
__POIOLC_VALUE__    = 3
__POIOLC_TYPE__     = 4

poiolc_opt_map = {
    '1' : (__POIOLC_STRENGTH__, "Podaj siłę trucizny: "),
    '2' : (__POIOLC_DURATION__, "Podaj czas działania trucizny: "),
    '3' : (__POIOLC_VALUE__, "Podaj wartość trucizny: "),
    '4' : (__POIOLC_TYPE__, "Wybierz typ trucizny: ")
    }

def poiolc_menu(sock, data):
    '''display our menu interface.'''
    buf = ['{g1) Siła           : {c' + str(data.strength),
           '{g2) Czas działania : {c' + str(data.duration),
           '{g3) Wartość        : {c' + str(data.value),
           '{g4) Typ            : {c' + data.type]
    sock.send('\r\n'.join(buf))

def poiolc_chooser(sock, data, option):
    '''figure out what we want to edit, and display a prompt.'''
    if not poiolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = poiolc_opt_map[option]

    if ret == __POIOLC_TYPE__:
        utils.olc_display_table(sock, poison_types, 4)
    sock.send_raw(mssg)
    return ret

def poiolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to.'''
    try:
        arg = int(arg)
    except:
        return False

    if choice == __POIOLC_STRENGTH__:
        data.strength = max(int(arg), 1)
    elif choice == __POIOLC_DURATION__:
        data.duration = max(int(arg), 1)
    elif choice == __POIOLC_VALUE__:
        data.value = max(int(arg), 1)
    elif choice == __POIOLC_TYPE__:
        if arg < 0 or arg >= len(poison_types):
            return False
        data.type = poison_types[arg]
    else:
        return False
    return True

def poiolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    buf = 'me.get_type_data("poison").strength = ' + str(data.strength) + '\n'
    buf += 'me.get_type_data("poison").duration = ' + str(data.duration) + '\n'
    buf += 'me.get_type_data("poison").value = ' + str(data.value) + '\n'
    buf += 'me.get_type_data("poison").type = "' + data.type + '"\n'
    return buf

################################################################################
# game commands
################################################################################
def cmd_poison(ch, cmd, arg):
    '''Usage: zatruj <poison> <object>

    Try to poison selected object with selected poison.'''
    if len(arg) == 0:
        ch.send('Prawidłowa składnia to: zatruj <trucizna> <rzecz>.')
        return
    args = arg.split()
    if len(args) < 2:
        ch.send('Podaj której trucizny chcesz użyć na którym przedmiocie.')
        return
    try:
        poison, = mud.parse_args(ch, True, cmd, args[0], 'obj.inv')
        item, = mud.parse_args(ch, True, cmd, args[1], 'obj.inv.eq')
    except:
        return
    if not poison.istype('poison'):
        ch.send(poison.name + ' nie jest trucizną.')
        return
    canPoison = False
    if not item.istype('weapon'):
        if item.istype('ammo'):
            if item.get_type_data('ammo').type in ['strzała', 'bełt', 'strzałka']:
                canPoison = True
    else:
        canPoison = True
    if not canPoison:
        ch.send('Nie możesz zatruć tego przedmiotu.')
        return
    item.settype('poison')
    item.get_type_data('poison').strength = poison.get_type_data('poison').strength
    item.get_type_data('poison').duration = poison.get_type_data('poison').duration
    item.get_type_data('poison').value = poison.get_type_data('poison').value
    item.get_type_data('poison').type = poison.get_type_data('poison').type
    mud.extract(poison)
    ch.send('Zatrułeś ' + item.name + '.')

################################################################################
# initialization
################################################################################

#initialize our item data, and its OLC
mudsys.item_add_type(PoisonData.__item_type__, PoisonData)
olc.item_add_olc(PoisonData.__item_type__,
                 poiolc_menu, poiolc_chooser, poiolc_parser,
                 None, poiolc_to_proto)

mudsys.add_cmd('zatruj', None, cmd_poison, 'player', False)
