# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
prompt.py

Show prompt to user with informations about character.
At this moment show only health and fatigue status.
'''

import mudsys, mud, socket, char, hooks, odmiana, utils

################################################################################
# Functions
################################################################################

def encode_text(text, encoding):
    '''Encode selected text.'''
    if encoding == 'brak':
        oldchars = ['ą', 'ć', 'ę', 'ó', 'ś', 'ł', 'ż', 'ź', 'ń']
        newchars = ['a', 'c', 'e', 'o', 's', 'l', 'z', 'z', 'n']
        i = 0
        for char in oldchars:
            text = text.replace(char, newchars[i])
            i += 1
    else:
        text = text.decode('utf-8')
        text = text.encode(encoding)
    return text

def new_prompt(sock):
    '''Function override default prompt.'''
    if sock.state != 'playing':
        return
    sock.send_raw("\r\n[" + utils.char_status(sock.ch, 'health', False, True) + "{n] [" + utils.char_status(sock.ch, 'fatigue', False, True) + "{n]")

################################################################################
# Hooks
################################################################################
def show_prompt_hook(info):
    '''Change text encoding in prompt if needed.'''
    sock, = hooks.parse_info(info)
    if sock.account == None:
        return
    if sock.aux('misc').encoding != 'utf-8':
        sock.outbound_text = encode_text(sock.outbound_text, sock.aux('misc').encoding)

def encoding_hook(info):
    '''Change text encoding if needed.'''
    sock, = hooks.parse_info(info)
    if sock.account == None:
        return
    if sock.ch != None:
        sock.outbound_text = odmiana.zastap_odmiane(sock.outbound_text, sock.ch)
    if sock.aux('misc').encoding != 'utf-8':
        sock.outbound_text = encode_text(sock.outbound_text, sock.aux('misc').encoding)

def inbound_hook(info):
    '''Change text encoding of incoming text if needed.'''
    sock, = hooks.parse_info(info)
    if sock.account == None:
        return
    if sock.aux('misc').encoding != 'utf-8':
        text = sock.inbound_text
        if sock.aux('misc').encoding != 'brak':
            text = text.decode(sock.aux('misc').encoding)
            text = text.encode('utf-8')
        sock.inbound_text = text

################################################################################
# initializing and unloading our hooks
################################################################################
hooks.add("process_outbound_prompt", show_prompt_hook)
hooks.add("process_outbound_text", encoding_hook)
hooks.add("receive_text", inbound_hook)
mudsys.show_prompt = new_prompt

def __unload__():
    '''deatach module from server'''
    hooks.remove("process_outbound_prompt", show_prompt_hook)
    hooks.remove("process_outboud_text", encoding_hook)
    hooks.remove("receive_text", inbound_hook)
