# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
randomenc.py

Provide functions and data structure for random encounters.
'''

import auxiliary, olc, storage, random, utils, hooks, zone, event, char, mud

################################################################################
# random encounters type definition
################################################################################

security = {'safe' : 0, 'peace' : 10, 'danger' : 30, 'war' : 50, 'deadly' : 70}

class RandomEncData:
    '''Data for random encounters.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.security = 'safe'
        self.enemies = []
        self.actions = []
        self.amount = 1

        if set != None:
            if len(set.readString('security')) > 0:
                self.security = set.readString('security')
                self.enemies = set.readString('enemies').split(';')
                self.actions = set.readString('actions').split(';')
                if set.readInt('amount') > 0:
                    self.amount = set.readInt('amount')

    def copyTo(self, to):
        to.security = self.security
        to.enemies = self.enemies
        to.actions = self.actions
        to.amount = self.amount

    def copy(self):
        newData = RandomEncData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('security', self.security)
        set.storeString('enemies', ';'.join(self.enemies))
        set.storeString('actions', ';'.join(self.actions))
        set.storeInt('amount', self.amount)
        return set

################################################################################
# Random Encounters OLC
################################################################################
__RANDENC_SECURITY__ = 1
__RANDENC_ENEMIES__  = 2
__RANDENC_ACTIONS__  = 3
__RANDENC_AMOUNT__ = 4

randencolc_opt_map = {
    '1' : (__RANDENC_SECURITY__, "Wybierz poziom bezpieczeństwa strefy: "),
    '2' : (__RANDENC_ENEMIES__, "Podaj listę przeciwników, oddzielonych średnikiem: "),
    '3' : (__RANDENC_ACTIONS__, "Podaj komendy dla przeciwników, oddzielone średnikiem: "),
    '4' : (__RANDENC_AMOUNT__, "Podaj maksymalną ilość przeciwników jaka może się pojawić: ")
    }

def randencolc_menu(sock, data):
    '''Display random encounters settings.'''
    buf = []
    buf.append('{g1) Bezpieczeństwo   : {c' + data.aux('randenc').security)
    buf.append('{g2) Przeciwnicy      : {c' + ';'.join(data.aux('randenc').enemies))
    buf.append('{g3) Akcje            : {c' + ';'.join(data.aux('randenc').actions))
    buf.append('{g4) Maksymalna ilość : {c' + str(data.aux('randenc').amount))
    sock.send('\r\n'.join(buf))

def randencolc_chooser(sock, data, option):
    '''figure out what field we want to edit, and display prompt.'''
    if not randencolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = randencolc_opt_map[option]

    if ret == __RANDENC_SECURITY__:
        utils.olc_display_table(sock, security.keys(), 4)
    sock.send_raw(mssg)
    return ret

def randencolc_parser(sock, data, choice, arg):
    '''parse out what we want to change a specifield field to.'''
    if choice in [__RANDENC_SECURITY__, __RANDENC_AMOUNT__]:
        try:
            arg = int(arg)
        except:
            return False
    elif choice == __RANDENC_SECURITY__:
        if arg >= len(security):
            return False
        data.aux('randenc').security = security.keys()[arg]
    elif choice == __RANDENC_ENEMIES__:
        data.aux('randenc').enemies = arg.split(';')
    elif choice == __RANDENC_ACTIONS__:
        data.aux('randenc').actions = arg.split(';')
    elif choice == __RANDENC_AMOUNT__:
        data.aux('randenc').amount = max(int(arg), 1)
    else:
        return False
    return True

def randencolc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send('Bezpieczeństwo')

def randencolc_choice_exec(sock, data):
    '''Edit random encounters.'''
    olc.do_olc(sock, randencolc_menu, randencolc_chooser, randencolc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# events
################################################################################
def randenc_event(owner, tgt, action):
    '''event for start fight in random encounter.'''
    if owner.room != tgt.room:
        mud.extract(owner)
        return
    owner.act(action + ' ' + tgt.name)
    event.start_event(owner, 60, clean_event, None, '')

def clean_event(owner, void, none):
    '''clear random encounter if still exist.'''
    if owner != None:
        mud.extract(owner)

################################################################################
# game hooks
################################################################################

def randenc_hook(info):
    '''When player enter room, check did random encounter appear.'''
    ch, rm = hooks.parse_info(info)
    #no point in continue
    if ch.is_npc:
        return
    area = zone.Zone(rm.locale)
    if area.aux('randenc').security == 'safe':
        return
    rand = random.randint(0, 100)
    #random encounter appear
    if rand <= security[area.aux('randenc').security]:
        rand = random.randint(0, len(area.aux('randenc').enemies) - 1)
        proto = area.aux('randenc').enemies[rand]
        rand = random.randint(0, len(area.aux('randenc').actions) - 1)
        amount = random.randint(1, area.aux('randenc').amount)
        if amount == 1:
            enemy = char.load_mob(proto, rm)
            event.start_event(enemy, 5, randenc_event, ch, area.aux('randenc').actions[rand])
        else:
            leader = ''
            for i in range(0, amount - 1):
                enemy = char.load_mob(proto, rm)
                if i == 0:
                    enemy.setparty('me')
                    leader = enemy.name
                else:
                    enemy.setparty(leader)
                event.start_event(enemy, 5 + i, randenc_event, ch, area.aux('randenc').actions[rand])
        ch.send('Masz przeczucie, że coś jest nie tak...')

################################################################################
# initialization
################################################################################
auxiliary.install('randenc', RandomEncData, 'zone')

olc.extend('zedit', 'R', randencolc_extend_menu, randencolc_choice_exec, None, None, None)

hooks.add('enter', randenc_hook)

def __unload__():
    '''remove the hooks for random encounters.'''
    hooks.remove('enter', randenc_hook)
