# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
ranged.py

Ranged weapons and ammunitons items types.
'''
import olc, mudsys, auxiliary, mudsock, storage, mud, utils, hooks

################################################################################
# item types definition
################################################################################

ammo_types = []
ammo_subtypes = []

class RangedData:
    '''Data for ranged weapons.'''

    __item_type__ = 'ranged'

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.preparation = 5
        self.ammo = 'strzała'
        self.gunpowder = False
        self.skill = 'Łuki'
        self.loaded = 0
        self.powder = 0

        if set != None:
            self.preparation = set.readInt('preparation')
            self.ammo = set.readString('ammo')
            self.gunpowder = set.readBool('gunpowder')
            self.skill = set.readString('skill')
            self.loaded = set.readInt('loaded')
            self.powder = set.readInt('powder')

    def copyTo(self, to):
        to.preparation = self.preparation
        to.ammo = self.ammo
        to.gunpowder = self.gunpowder
        to.skill = self.skill
        to.loaded = self.loaded
        to.powder = self.powder

    def copy(self):
        newData = RangedData()
        self.copyTo(newData)

    def store(self):
        set = storage.StorageSet()
        set.storeInt('preparation', self.preparation)
        set.storeString('ammo', self.ammo)
        set.storeBool('gunpowder', self.gunpowder)
        set.storeString('skill', self.skill)
        set.storeInt('loaded', self.loaded)
        set.storeInt('powder', self.powder)
        return set

class AmmoData:
    '''Data for ammunition for ranged weapons.'''

    __item_type__ = 'ammo'

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.type = 'strzała'
        self.subtype = 'impaling'
        self.damage = 1

        if set != None:
            self.type = set.readString('type')
            self.subtype = set.readString('subtype')
            self.damage = set.readInt('damage')

    def copyTo(self, to):
        to.type = self.type
        to.subtype = self.subtype
        to.damage = self.damage

    def copy(self):
        newData = AmmoData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString('type', self.type)
        set.storeString('subtype', self.subtype)
        set.storeInt('damage', self.damage)
        return set

################################################################################
# functions
################################################################################
def ammo_manip(ch, uid, type = ''):
    '''If uid is 0, return ammunition or gunpowder, otherwise reset hidden value
       for item.'''
    ammo = None
    for item in ch.eq:
        if item.istype('container'):
            for item2 in item.contents:
                if len(type) > 0:
                    if item.istype('ammo'):
                        if item.get_type_data('ammo').type == type:
                            ammo = item
                            break
                else:
                    if uid == 0:
                        if item.name == 'proch strzelniczy':
                            ammo = item
                            break
                    else:
                        if item.uid == uid:
                            item.hidden = 0
                            ammo = item
                            break
    if ammo == None:
        for item in ch.inv:
            if len(type) > 0:
                if item.istype('ammo'):
                    if item.get_type_data('ammo').type == type:
                        ammo = item
                        break
            else:
                if uid == 0:
                    if item.name == 'proch strzelniczy':
                        ammo = item
                        break
                else:
                    if item.uid == uid:
                        item.hidden = 0
                        ammo = item
                        break
    return ammo


################################################################################
# Ranged OLC
################################################################################
__RANGEDOLC_PREPARE__    = 1
__RANGEDOLC_AMMO__       = 2
__RANGEDOLC_GUNPOWDER__  = 3
__RANGEDOLC_SKILL__      = 4

rangedolc_opt_map = {
    '1' : (__RANGEDOLC_PREPARE__, 'Wprowadź czas ładowania w sek: '),
    '2' : (__RANGEDOLC_AMMO__, "Wybierz rodzaj amunicji: "),
    '3' : (__RANGEDOLC_GUNPOWDER__, ''),
    '4' : (__RANGEDOLC_SKILL__, 'Wprowadź umiejętność: ')
    }

def rangedolc_menu(sock, data):
    '''display our menu interface.'''
    if data.gunpowder:
        gpowder = 'Tak'
    else:
        gpowder = 'Nie'
    buf = ['{g1) Czas ładowania w sek : {c' + str(data.preparation),
           '{g2) Typ amunicji         : {c' + data.ammo,
           '{g3) Wymaga prochu        : {c' + gpowder,
           '{g4) Umiejętność          : {c' + data.skill]
    sock.send('\r\n'.join(buf))

def rangedolc_chooser(sock, data, option):
    '''figure out what field we want to edit and display a prompt.'''
    if not rangedolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = rangedolc_opt_map[option]

    #change if weapon need gunpowder
    if ret == __RANGEDOLC_GUNPOWDER__:
        data.gunpowder = not data.gunpowder
        return olc.MENU_NOCHOICE

    #list ammunition types options
    if ret == __RANGEDOLC_AMMO__:
        utils.olc_display_table(sock, ammo_types, 4)
    sock.send_raw(mssg)
    return ret

def rangedolc_parser(sock, data, choice, arg):
    '''parse out what we want to change at specified field to.'''
    if choice != __RANGEDOLC_SKILL__:
        try:
            arg = int(arg)
        except:
            return

    if choice == __RANGEDOLC_PREPARE__:
        data.preparation = max(int(arg), 1)
    elif choice == __RANGEDOLC_AMMO__:
        arg = int(arg)
        if arg < 0 or arg >= len(ammo_types):
            return False
        data.ammo = ammo_types[arg]
    elif choice == __RANGEDOLC_SKILL__:
        data.skill = str(arg)
    else:
        return False
    return True

def rangedolc_to_proto(data):
    '''return a script that will generate relevant info to create this item.'''
    buf = 'me.get_type_data("ranged").preparation = ' + str(data.preparation) + '\n'
    buf += 'me.get_type_data("ranged").ammo = "' + data.ammo + '"\n'
    buf += 'me.get_type_data("ranged").gunpowder = ' + str(data.gunpowder) + '\n'
    buf += 'me.get_type_data("ranged").skill = "' + data.skill + '"\n'
    buf += 'me.get_type_data("ranged").loaded = ' + str(data.loaded) + '\n'
    buf += 'me.get_type_data("ranged").powder = ' + str(data.powder) + '\n'
    return buf

################################################################################
# Ammo OLC
################################################################################
__AMMOOLC_TYPE__    = 1
__AMMOOLC_SUBTYPE__ = 2
__AMMOOLC_DAMAGE__  = 3

ammoolc_opt_map = {
    '1' : (__AMMOOLC_TYPE__, 'Wybierz typ: '),
    '2' : (__AMMOOLC_SUBTYPE__, 'Wybierz podtyp: '),
    '3' : (__AMMOOLC_DAMAGE__, 'Wprowadź zadawane obrażenia: ')
    }

def ammoolc_menu(sock, data):
    '''display our menu interface.'''
    buf = ['{g1) Typ       : {c' + data.type,
           '{g2) Podtyp    : {c' + data.subtype,
           '{g3) Obrażenia : {c' + str(data.damage)]
    sock.send('\r\n'.join(buf))

def ammoolc_chooser(sock, data, option):
    '''figure out what we want to edit and display a prompt.'''
    if not ammoolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = ammoolc_opt_map[option]

    #list our types option
    if ret == __AMMOOLC_TYPE__:
        utils.olc_display_table(sock, ammo_types, 4)
    elif ret == __AMMOOLC_SUBTYPE__:
        utils.olc_display_table(sock, ammo_subtypes, 4)
    sock.send_raw(mssg)
    return ret

def ammoolc_parser(sock, data, choice, arg):
    '''parse out what we want to change at specified field to'''
    try:
        arg = int(arg)
    except:
        return

    if choice == __AMMOOLC_TYPE__:
        arg = int(arg)
        if arg < 0 or arg >= len(ammo_types):
            return False
        data.type = ammo_types[arg]
    elif choice == __AMMOOLC_SUBTYPE__:
        arg = int(arg)
        if arg < 0 or arg >= len(ammo_subtypes):
            return False
        data.subtype = ammo_subtypes[arg]
    elif choice == __AMMOOLC_DAMAGE__:
        data.damage = max(int(arg), 1)
    else:
        return False
    return True

def ammoolc_to_proto(data):
    '''return a script that will generate relevant info to create this item'''
    buf = 'me.get_type_data("ammo").type = "' + data.type + '"\n'
    buf += 'me.get_type_data("ammo").subtype = "' + data.subtype + '"\n'
    buf += 'me.get_type_data("ammo").damage = ' + str(data.damage) + '\n'
    return buf

################################################################################
# game hooks
################################################################################
def ranged_desc_info(info):
    '''appends ranged weapon informations about an object to the description
       buffer when person looks at it.'''
    obj, ch = hooks.parse_info(info)

    #no point in continuing
    if not obj.istype("ranged"):
        return

    if obj.get_type_data("ranged").loaded > 0:
        loaded = 'Obecnie jest załadowana.'
    else:
        loaded = 'Obecnie nie jest załadowana.'
    ammo = ['strzał', 'bełtów', 'kul', 'strzałek', 'pocisków']
    index = ammo_types.index(obj.get_type_data("ranged").ammo)

    desc = ' Jest to broń dystansowa, używająca ' + ammo[index] + ' jako amunicji. Wymaga umiejętności: ' + obj.get_type_data("ranged").skill + '. Ładowanie tej broni trwa ' + str(obj.get_type_data("ranged").preparation) + ' sekund. ' + loaded

    ch.look_buf += desc

def ammo_desc_info(info):
    '''appends ammo informations about an object to the description buffer when
    person looks at it.'''
    obj, ch = hooks.parse_info(info)

    #no point in continuing
    if not obj.istype("ammo"):
        return

    if obj.get_type_data("ammo").subtype == "cut":
        dtype = "tnące"
    elif obj.get_type_data("ammo").subtype == "impaling":
        dtype = "przebijające"
    elif obj.get_type_data("ammo").subtype == "crush":
        dtype = "miażdżone"

    desc = ' Jest to amunicja typu ' + obj.get_type_data("ammo").type + ' do broni dystansowej, zdająca rany ' + dtype + '. Zwiększa zadawane obrażenia o ' + str(obj.get_type_data("ammo").damage) + '%.'

    ch.look_buf += desc

def ranged_unequip_hook(info):
    '''Check if we try unequip loaded ranged weapon. If yes, unload it or stop
    loading.'''
    ch, obj = hooks.parse_info(info)

    #no point in continue
    if obj.worn_locs.find('dzierżone') == -1 or not obj.istype('ranged'):
        return

    #if we loading this weapon, stop it.
    loading = False
    for cooldown in ch.aux('cooldowns').names:
        if cooldown.startwith('+Załadował'):
            index = ch.aux('cooldowns').names.index(cooldown)
            del ch.aux('cooldowns').names[index]
            del ch.aux('cooldowns').times[index]
            loading = True
    #unload weapon
    ammo = ammo_manip(ch, obj.get_type_data('ranged').loaded)
    ammo_manip(ch, obj.get_type_data('ranged').powder)
    obj.get_type_data('ranged').loaded = 0
    obj.get_type_data('ranged').powder = 0
    if loading:
        ch.send("Przestajesz ładować " + obj.name + "$a.")
    else:
        if ammo != None:
            ch.send("Rozładowujesz " + obj.name + "$a.")

def ranged_load_hook(info):
    '''Send informations about loaded weapon.'''
    ch, mssg = hooks.parse_info(info)

    #no point in continue
    if not mssg.startswith('Załadował'):
        return

    if ch.sex == 'mężczyzna':
        suffix1 = 'eś'
        suffix2 = ''
    else:
        suffix1 = 'aś'
        suffix2 = 'a'

    parts = mssg.split()
    ch.send(parts[0] + suffix1 + ' ' + ' '.join(parts[1:]))
    mud.message(ch, None, None, None, False, 'to_room', '$n' + parts[0].lower() + suffix2 + ' ' + ' '.join(parts[1:]))

################################################################################
# game commands
################################################################################
def cmd_load(ch, cmd, arg):
    '''Usage: zaladuj [pocisk] [z] [pojemnik]

    Load ranged weapon in hand with selected ammo. If no ammo specified, get first
    from inventory.'''
    #did we have ranged weapon in hands?
    weapon = ch.get_equip('dzierżone')
    if weapon == None or not weapon.istype('ranged'):
        ch.send('Nie masz broni dystansowej w rękach.')
        return
    if weapon.get_type_data('ranged').loaded > 0:
        ch.send('Ta broń jest już załadowana albo jest ładowana.')
        return
    ammo = None
    #if no ammo specified, get it from inventory or from container in equipment
    if len(arg) == 0:
        ammo = ammo_manip(ch, 0, weapon.get_type_data('ranged').ammo)
        if ammo == None:
            ch.send("Nie masz amunicji do tej broni.")
            return
    else:
        try:
            ammo, cont = mud.parse_args(ch, True, cmd, arg, 'obj.inv | [z] obj.inv.eq')
        except:
            return
    #if weapon need gunpowder, we should use it too
    if weapon.get_type_data('ranged').gunpowder:
        gunpowder = ammo_manip(ch, 0, '')
        if gunpowder == None:
            ch.send("Nie masz prochu strzelniczego aby załadować tę broń.")
            return
        weapon.get_type_data('ranged').powder = gunpowder.uid
        gunpowder.hidden = 10000
    weapon.get_type_data('ranged').loaded = ammo.uid
    ammo.hidden = 10000
    ch.aux('cooldowns').names.append('+Załadował ' + weapon.name + '$a.')
    ch.aux('cooldowns').times.append(weapon.get_type_data('ranged').preparation)
    ch.send("Rozpoczynasz ładowanie broni.")

################################################################################
# initialization
################################################################################

#initialize our items data, and its OLC
mudsys.item_add_type(RangedData.__item_type__, RangedData)
olc.item_add_olc(RangedData.__item_type__,
                 rangedolc_menu, rangedolc_chooser, rangedolc_parser,
                 None, rangedolc_to_proto)
mudsys.item_add_type(AmmoData.__item_type__, AmmoData)
olc.item_add_olc(AmmoData.__item_type__,
                 ammoolc_menu, ammoolc_chooser, ammoolc_parser,
                 None, ammoolc_to_proto)

#initialize some default ammunition types and subtypes
ammo_types.append('strzała')
ammo_types.append('bełt')
ammo_types.append('kula')
ammo_types.append('strzałka')
ammo_types.append('pocisk')
ammo_subtypes.append('impaling')
ammo_subtypes.append('cut')
ammo_subtypes.append('crush')

#display hooks for ranged weapons
hooks.add("append_obj_desc", ranged_desc_info)
hooks.add("append_obj_desc", ammo_desc_info)
hooks.add("unequip", ranged_unequip_hook)
hooks.add('cooldown', ranged_load_hook)

#add game commands
mudsys.add_cmd("zaladuj", None, cmd_load, "player", False)

def __unload__():
    '''remove all hooks.'''
    hooks.remove('append_obj_desc', ranged_desc_info)
    hooks.remove('append_obj_desc', ammo_desc_info)
    hooks.remove('unequip', ranged_unequip_hook)
    hooks.remove('cooldown', ranged_load_hook)
