# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
riddles.py

Functions for random riddles.
'''
import storage, hashlib, mudsys, random, mud

################################################################################
# dreams data type definition
################################################################################
__riddles_file__ = '../lib/misc/riddles'
__riddles__ = []

class Riddle:
    '''Implements one riddle.'''
    def __init__(self, body, answer, set = None):
        '''Standard class constructor.'''
        self.body = body
        self.answer = answer
        if set != None:
            self.body = set.readString('body')
            self.answer = set.readString('answer')

    def store(self):
        '''return a storage set representation of the riddle.'''
        set = storage.StorageSet()
        set.storeString('body', self.body)
        set.storeString('answer', self.answer)
        return set

################################################################################
# interaction functions
################################################################################
def save_riddles():
    '''save all of the riddles to a file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList('riddles', list)
    for val in __riddles__:
        one_set = storage.StorageSet()
        one_set.storeSet('riddle', val.store())
        list.add(one_set)
    set.write(__riddles_file__)
    set.close()

def load_riddles():
    '''read all of the riddles from file.'''
    set = storage.StorageSet(__riddles_file__)
    for riddle in set.readList('riddles').sets():
        __riddles__.append(Riddle(None, None, riddle.readSet('riddle')))
    set.close()
    return

def set_riddle(self):
    '''set riddle on object.'''
    rand = random.randint(0, len(__riddles__) - 1)
    self.setvar('riddle', str(rand))
    text = mud.format_string('Powoli czytasz napis: "' + __riddles__[rand].body + '" Domyślasz się, że musisz powiedzieć odpowiedź aby rozwiązać tę zagadkę.')
    self.edesc('napis, tekst, zagadka', text)

def check_answer(self, answer):
    '''return True if answer is valid, otherwise False.'''
    index = int(self.getvar('riddle'))
    answer = hashlib.md5(answer).hexdigest()
    if answer == __riddles__[index].answer:
        self.delvar('riddle')
        self.deledesc('napis, tekst, zagadka')
        return True
    return False

################################################################################
# game commands
################################################################################
def cmd_riddles(ch, cmd, arg):
    '''Usage: zagadki <dodaj | usun | pokaz> [number | answer]

    Add, delete or show riddles.'''
    if len(arg) == 0:
        ch.send('Prawidłowa składnia to zagadki <dodaj | usun | pokaz> [numer | odpowiedź]')
        return
    options = arg.split()
    #show list of riddles
    if options[0] == 'pokaz':
        if len(__riddles__) == 0:
            ch.send('Nie ma jeszcze dodanych zagadek.')
            return
        i = 1
        buf = []
        for riddle in __riddles__:
            buf.append(str(i) + ": " + riddle.body)
            i += 1
        ch.page('\r\n'.join(buf))
    #add a riddle
    elif options[0] == 'dodaj':
        if ch.notepad == '':
            ch.send('Twój notatnik jest pusty. Spróbuj coś napisać.')
            return
        if len(options) < 2:
            ch.send('Musisz podać odpowiedź do zagadki.')
            return
        answer = hashlib.md5(options[1]).hexdigest()
        __riddles__.append(Riddle(ch.notepad, answer))
        save_riddles()
        ch.send("Zagadka została dodana.")
    #delete a riddle
    elif options[0] == 'usun':
        if len(options) < 2:
            ch.send('Podaj numer zagadki do usunięcia.')
            return
        try:
            number = int(options[1]) - 1
        except:
            ch.send('To nie jest liczba.')
            return
        if number < 0 or number > len(__riddles__) - 1:
            ch.send('Nie ma takiej zagadki.')
            return
        del __riddles__[number]
        save_riddles()
        ch.send('Zagadka została usunięta')

################################################################################
# initialization
################################################################################
load_riddles()

mudsys.add_cmd('zagadki', None, cmd_riddles, 'admin', False)

mudsys.add_obj_method('setriddle', set_riddle)
mudsys.add_obj_method('checkanswer', check_answer)
