# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
score.py

Player informations - stats, skills, etc
'''

import mudsys, char, mud, math

################################################################################
# game commands
################################################################################
def cmd_score(ch, cmd, arg):
    '''Show player stats.'''
    name = ''
    if len(ch.aux('title').pretitle) > 0:
        name = ch.aux('title').pretitle + ' '
    name += ch.name
    if len(ch.aux('title').title) > 0:
        name += ' ' + ch.aux('title').title
    ch.send("Imię: " + name)
    #count player age
    hours = int((ch.age / 3600)) % 24
    days = int((ch.age / 3600) / 24)
    buf = []
    buf.append("Wiek: %d dni %d godzin(y)" % (days, hours))
    buf.append("Płeć: " + ch.sex)
    buf.append("Rasa: " + ch.race)
    buf.append("Charakter: " + ch.aux('alignment').desc)
    buf.append("Wyznanie: " + ch.aux('faith').deity)
    if len(ch.aux('faith').desc) > 0:
        buf.append("Stosunek bóstwa do ciebie: " + ch.aux('faith').desc)
    descs = {'Siła' : ['{rbardzo słaby{n', '{ysłaby{n', '{cprzeciętny{n', '{bsilny{n', '{gbardzo silny{n'],
             'Zręczność' : ['{rniezdara{n', '{yniezręczny{n', '{cprzeciętny{n', '{bzręczny{n', '{gbardzo zręczny{n'],
             'Inteligencja' : ['{rzagubiony{n', '{yniezbyt inteligentny{n', '{cinteligentny{n', '{bbystry{n', '{gbardzo bystry{n'],
             'Kondycja' : ['{rchorowity{n', '{ysłaby{n', '{cprzeciętny{n', '{bwytrzymały{n', '{gbardzo wytrzymały{n']}
    for i in range(0, len(ch.aux('stats').names)):
        name = ch.aux('stats').names[i]
        value = ch.aux('stats').values[i]
        if i < 4:
            if value < 10:
                level = descs[name][0]
            elif value > 9 and value < 20:
                level = descs[name][1]
            elif value > 19 and value < 30:
                level = descs[name][2]
            elif value > 29 and value < 40:
                level = descs[name][3]
            else:
                level = descs[name][4]
            buf.append(name + ": " + level)
        else:
            stat_percent = (value / ch.aux('stats').secondvalues[i]) * 100
            buf.append("%s: %d%%" % (name, stat_percent))
    buf.append('')
    buf.append('')
    buf.append('Aby zobaczyć znane umiejętności, wpisz umiejetnosci.')
    buf.append('Aby zobaczyć znane czary, wpisz czary.')
    buf.append('Aby zobaczyć znane plany, wpisz plany.')
    ch.send('\r\n'.join(buf))

def cmd_skills(ch, cmd, arg):
    '''Show player skills.'''
    if len(ch.aux('skills').names) == 0:
        ch.send("Brak umiejętności")
    else:
        descs = ['{rnowicjusz{n', '{ypoczątkujący{n', '{czdolny{n', '{bzawodowiec{n', '{gmistrz{n']
        buf = []
        for i in range(0, len(ch.aux('skills').names)):
            value = ch.aux('skills').levels[i]
            if value < 20:
                level = descs[0]
            elif value > 19 and value < 40:
                level = descs[1]
            elif value > 39 and value < 60:
                level = descs[2]
            elif value > 59 and value < 80:
                level = descs[3]
            else:
                level = descs[4]
            buf.append(ch.aux('skills').names[i] + ': ' + level)
        ch.send('\r\n'.join(buf))

def cmd_spells(ch, cmd, arg):
    '''Show player spells.'''
    if len(ch.aux('spells').names) == 0:
        ch.send('Nie znasz jakichkolwiek czarów.')
    else:
        descs = ['{rnowicjusz{n', '{ypoczątkujący{n', '{czdolny{n', '{bzawodowiec{n', '{gmistrz{n']
        buf = []
        for i in range(0, len(ch.aux('spells').names)):
            value = ch.aux('spells').levels[i]
            if value < 20:
                level = descs[0]
            elif value > 19 and value < 40:
                level = descs[1]
            elif value > 39 and value < 60:
                level = descs[2]
            elif value > 59 and value < 80:
                level = descs[3]
            else:
                level = descs[4]
            buf.append(ch.aux('spells').names[i] + ': ' + level)
        ch.send('\r\n'.join(buf))

def cmd_plans(ch, cmd, arg):
    '''Show player plans.'''
    if len(ch.aux('plans').names) == 0:
        ch.send('Nie znasz jakichkolwiek planów.')
    else:
        plans = {}
        for i in range(0, len(ch.aux('plans').names)):
            if ch.aux('plans').skills[i] in plans.keys():
                plans[ch.aux('plans').skills[i]].append(ch.aux('plans').names[i])
            else:
                plans[ch.aux('plans').skills[i]] = [ch.aux('plans').names[i]]
        buf = []
        for skill in plans.keys():
            buf.append(skill)
            i = 1
            for plan in plans[skill]:
                index = ch.aux('plans').names.index(plan)
                if ch.aux('plans').learned[index] == 'Y':
                    learned = '(nauczony)'
                else:
                    learned = '(studiowany)'
                buf.append(str(i) + ') ' + plan + ' ' + learned)
                i += 1
        ch.send('\r\n'.join(buf))

################################################################################
# initialization
################################################################################

#add commands
mudsys.add_cmd("karta", None, cmd_score, "player", False)
mudsys.add_cmd("umiejetnosci", None, cmd_skills, "player", False)
mudsys.add_cmd("czary", None, cmd_spells, "player", False)
mudsys.add_cmd("plany", None, cmd_plans, "player", False)
