# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
spells.py

Provide data and functions for handle players spells.
'''

import mudsys, mud, auxiliary, storage, olc, utils, random, gainlevel, fight, hooks
from skills import SkillsData

################################################################################
# spells data type definition
################################################################################
__spells_file__ = "../lib/misc/spells"
__spells__ = []
spell_names = []

class Spell:
    '''Implements one spell.'''
    def __init__(self, name, set = None):
        '''Standard class constructor.'''
        self.name = name
        self.group = 'battle'
        self.target = 'char'
        self.area = False
        self.duration = 0
        self.difficult = 0
        self.cost = 10
        self.value = 0
        self.type = 'damage.crush'
        self.needs = 'spells.0'
        if set != None:
            self.name = set.readString('name')
            self.group = set.readString('group')
            self.target = set.readString('target')
            self.area = set.readBool('area')
            self.duration = set.readInt('duration')
            self.difficult = set.readInt('difficult')
            self.cost = set.readInt('cost')
            self.value = set.readInt('value')
            self.type = set.readString('type')
            self.needs = set.readString('needs')

    def store(self):
        '''return a storage set representation of the spell.'''
        set = storage.StorageSet()
        set.storeString('name', self.name)
        set.storeString('group', self.group)
        set.storeString('target', self.target)
        set.storeBool('area', self.area)
        set.storeInt('duration', self.duration)
        set.storeInt('difficult', self.difficult)
        set.storeInt('cost', self.cost)
        set.storeInt('value', self.value)
        set.storeString('type', self.type)
        set.storeString('needs', self.needs)
        return set

################################################################################
# interaction functions
################################################################################
def save_spells(spell = None):
    '''save all of the spells to file.'''
    set = storage.StorageSet()
    list = storage.StorageList()
    set.storeList("spells", list)
    for val in __spells__:
        one_set = storage.StorageSet()
        one_set.storeSet("spell", val.store())
        list.add(one_set)
    set.write(__spells_file__)
    set.close()

def load_spells():
    '''read all of the spells from file.'''
    set = storage.StorageSet(__spells_file__)
    for spell in set.readList("spells").sets():
        one_spell = spell.readSet("spell")
        __spells__.append(Spell(None, one_spell))
        spell_names.append(one_spell.readString('name'))
    set.close()
    return

def get_spell(spellname):
    '''function return spell statistics.'''
    index = spell_names.index(spellname)
    return __spells__[index]

def add_spell(self, name):
    '''function add new spell to character.'''
    self.aux('spells').names.append(name)
    self.aux('spells').levels.append(0)
    self.aux('spells').baselevels.append(0)
    self.aux('spells').exp.append(0)

################################################################################
# Spells OLC
################################################################################
SPELL_NEW = 1
SPELL_EDIT = 2
SPELL_DELETE = 3

def spellsolc_menu(sock, data):
    '''Display all spells and values for they.'''
    buf = []
    if len(data.aux('spells').names) == 0:
        buf.append('Brak czarów.')
    else:
        for i in range(len(data.aux('spells').names)):
            buf.append("%s: %d" % (data.aux('spells').names[i], data.aux('spells').levels[i]))
    buf.append('')
    buf.append('')
    buf.append('N) Nowy czar')
    if len(data.aux('spells').names) > 0:
        buf.append('E) Edytuj czar')
        buf.append('D) Skasuj czar')
    sock.send('\r\n'.join(buf))

def spellsolc_chooser(sock, data, option):
    '''Try pick spell to set.'''
    options = ['n', 'e', 'd']
    option = option.lower()
    if not option in options:
        return olc.MENU_CHOICE_INVALID
    if option == 'n':
        if len(spell_names) == 0:
            sock.send("Nie ma dostępnych zaklęć.")
            return olc.MENU_CHOICE_INVALID
        utils.olc_display_table(sock, spell_names, 4)
        sock.send("Wybierz dostępne zaklęcie: ")
        return SPELL_NEW
    else:
        if len(data.aux('spells').names) == 0:
            return olc.MENU_CHOICE_INVALID
        else:
            if option == 'd':
                sock.send("Nazwa zaklęcia do skasowania: ")
                return SPELL_DELETE
            else:
                sock.send("Nazwa zaklęcia do edycji: ")
                return SPELL_EDIT

def spellsolc_parser(sock, mobile, choice, arg):
    '''Set new spell or level for it.'''
    if choice == SPELL_EDIT or choice == SPELL_DELETE:
        if arg not in mobile.aux('spells').names:
            sock.send('Ta postać nie zna takiego zaklęcia.')
            return False
        i = mobile.aux('spells').names.index(arg)
    if choice == SPELL_DELETE:
        del mobile.aux('spells').names[i]
        del mobile.aux('spells').levels[i]
        del mobile.aux('spells').baselevels[i]
        del mobile.aux('spells').exp[i]
    elif choice == SPELL_EDIT:
        olc.do_olc(sock, spellolc_menu, spellolc_chooser, None, None, [mobile, i])
    elif choice == SPELL_NEW:
        try:
            arg = int(arg)
        except:
            return False
        if arg < 0 or arg >= len(spell_names):
            return False
        mobile.addspell(spell_names[arg])
        i = mobile.aux('spells').names.index(spell_names[arg])
        olc.do_olc(sock, spellolc_menu, spellolc_chooser, None, None, [mobile, i])
    return True

def spellsolc_to_proto(thing):
    '''Save spells.'''
    names = '['
    length = len(thing.aux('spells').names)
    if length > 0:
        for i in range(0, length):
            names += "'" + thing.aux('spells').names[i] + "'"
            if i < length - 1:
                names += ', '
    names += ']'
    levels = '['
    if length > 0:
        for i in range(0, length):
            levels += str(thing.aux('spells').levels[i])
            if i < length - 1:
                levels += ', '
    levels += ']'
    baselevels = '['
    if length > 0:
        for i in range(0, length):
            baselevels += str(thing.aux('spells').baselevels[i])
            if i < length - 1:
                baselevels += ', '
    baselevels += ']'
    buf = 'me.aux("spells").names = %s\n' % names
    buf += 'me.aux("spells").levels = %s\n' % levels
    buf += 'me.aux("spells").baselevels = %s\n' % baselevels
    return buf

def spellsolc_extend_menu(sock, data):
    '''Display option in main menu.'''
    sock.send("Edytuj zaklęcia")

def spellsolc_choice_exec(sock, data):
    '''Edit spells.'''
    olc.do_olc(sock, spellsolc_menu, spellsolc_chooser, spellsolc_parser, None, data)
    return olc.MENU_NOCHOICE

def spellolc_menu(sock, data):
    '''Show level of spell.'''
    buf = ['Czar: ' + data[0].aux('spells').names[data[1]]]
    buf.append('Poziom: %d' % data[0].aux('spells').levels[data[1]])
    buf.append('Nowy poziom czaru: ')
    sock.send('\r\n'.join(buf))

def spellolc_chooser(sock, data, option):
    '''Set new level for selected spell.'''
    try:
        opt = int(option)
        if opt < 0:
            return olc.MENU_CHOICE_INVALID
        data[0].aux('spells').levels[data[1]] = opt
        return olc.MENU_NOCHOICE
    except:
        return olc.MENU_CHOICE_INVALID

__SPELL_GROUP__     = 1
__SPELL_TARGET__    = 2
__SPELL_AREA__      = 3
__SPELL_DURATION__  = 4
__SPELL_DIFFICULT__ = 5
__SPELL_COST__      = 6
__SPELL_VALUE__     = 7
__SPELL_TYPE__      = 8
__SPELL_NEEDS__     = 9

spellolc_opt_map = {
    '1' : (__SPELL_GROUP__, 'Wybierz rodzaj czaru: '),
    '2' : (__SPELL_TARGET__, 'Wybierz rodzaj celu czaru: '),
    '3' : (__SPELL_AREA__, ''),
    '4' : (__SPELL_DURATION__, 'Podaj długość działania czaru: '),
    '5' : (__SPELL_DIFFICULT__, 'Podaj trudność rzucenia czaru: '),
    '6' : (__SPELL_COST__, 'Podaj koszt zmęczenia: '),
    '7' : (__SPELL_VALUE__, 'Podaj wartość efektu czaru: '),
    '8' : (__SPELL_TYPE__, 'Podaj efekt czaru: '),
    '9' : (__SPELL_NEEDS__, 'Podaj wymagania czaru: ')
    }

spell_groups = ['battle', 'other']
spell_targets = ['char', 'self', 'object']

def sedit_menu(sock, data):
    '''Display main menu for add/modify spell.'''
    if data.area:
        area = 'Tak'
    else:
        area = 'Nie'
    buf = ['{gNazwa            : ' + data.name,
           '{g1)Rodzaj         : {c' + data.group,
           '{g2)Cel            : {c' + data.target,
           '{g3)Obszar         : {c' + area,
           '{g4)Czas działania : {c' + str(data.duration),
           '{g5)Trudność       : {c' + str(data.difficult),
           '{g6)Zmęczenie      : {c' + str(data.cost),
           '{g7)Wartość efektu : {c' + str(data.value),
           '{g8)Efekt          : {c' + data.type,
           '{g9)Wymagania      : {c' + data.needs]
    sock.send('\r\n'.join(buf))

def sedit_chooser(sock, data, option):
    '''figure out what we want to edit in spell and display a prompt.'''
    if not spellolc_opt_map.has_key(option):
        return olc.MENU_CHOICE_INVALID
    ret, mssg = spellolc_opt_map[option]

    #list available group options
    if ret == __SPELL_GROUP__:
        utils.olc_display_table(sock, spell_groups, 4)

    #list available target options
    if ret == __SPELL_TARGET__:
        utils.olc_display_table(sock, spell_targets, 4)

    #switch area effect
    if ret == __SPELL_AREA__:
        data.area = not data.area
        return olc.MENU_NOCHOICE

    sock.send_raw(mssg)
    return ret

def sedit_parser(sock, data, choice, arg):
    '''parse out what we want to change a specified field to in spell.'''
    options = [__SPELL_GROUP__, __SPELL_TARGET__, __SPELL_DURATION__, __SPELL_COST__, __SPELL_VALUE__, __SPELL_DIFFICULT__]
    if choice in options:
        try:
            arg = int(arg)
        except:
            return False
    if choice == __SPELL_GROUP__:
        arg = int(arg)
        if arg < 0 or arg >= len(spell_groups):
            return False
        data.group = spell_groups[arg]
    elif choice == __SPELL_TARGET__:
        arg = int(arg)
        if arg < 0 or arg >= len(spell_targets):
            return False
        data.target = spell_targets[arg]
    elif choice == __SPELL_DURATION__:
        data.duration = max(int(arg), 1)
    elif choice == __SPELL_DIFFICULT__:
        data.difficult = int(arg)
    elif choice == __SPELL_COST__:
        data.cost = max(int(arg), 1)
    elif choice == __SPELL_VALUE__:
        data.value = int(arg)
    elif choice == __SPELL_TYPE__:
        data.type = arg
    elif choice == __SPELL_NEEDS__:
        data.needs = arg
    else:
        return False
    return True

################################################################################
# game hooks
################################################################################
def spell_fading_hook(info):
    '''Send informations about end of spell.'''

    ch, mssg = hooks.parse_info(info)

    #no point in continue
    if not mssg.startswith('Czar '):
        return
    
    parts = mssg.split()
    spellname = ' '.join(parts[1:-3])
    affect = spellname + ';' + parts[-1]
    if not affect in ch.aux('affects').sources:
        return
    mssg = parts[0:-1]
    index = ch.aux('affects').sources.index(affect)
    del ch.aux('affects').names[index]
    del ch.aux('affects').types[index]
    del ch.aux('affects').sources[index]
    index = spell_names.index(spellname)
    spelltype = __spells__[index].type.split('.')
    #stats bonuses spells
    if spelltype[0] == 'stat':
        if spelltype[1] == 'values':
            ch.aux('stats').values[number] += __spells__[sindex].value
        else:
            ch.aux('stats').secondvalues[number] += __spells__[sindex].value
    #skills bonuses spells
    elif spelltype[0] == 'skills':
        for vict in tgt:
            if spelltype[1] not in vict.aux('skills').names:
                vict.addskill(spelltype[1], 0, __spells__[sindex].value)
                number = vict.aux('skills').names.index(spelltype[1])
            else:
                number = vict.aux('skills').names.index(spelltype[1])
                vict.aux('skills').values[number] += __spells_[sindex].value
    ch.send(mssg)
    
################################################################################
# game commands
################################################################################
def cmd_sedit(ch, cmd, arg):
    '''Usage: sedit <nazwa czaru>

    Add or edit spell.'''
    if len(arg) == 0:
        ch.send("Podaj nazwę czaru.")
        return
    if arg in spell_names:
        index = spell_names.index(arg)
        spell = __spells__[index]
    else:
        spell = Spell(arg)
        spell_names.append(arg)
        __spells__.append(spell)
    olc.do_olc(ch.sock, sedit_menu, sedit_chooser, sedit_parser, save_spells, spell)

def cmd_sdelete(ch, cmd, arg):
    '''Usage: sdelete <nazwa czaru>

    Delete spell.'''
    if len(arg) == 0:
        ch.send('Podaj nazwę czaru do skasowania.')
        return
    if arg in spell_names:
        index = spell_names.index(arg)
        del spell_names[index]
        del __spells__[index]
        save_spells()
        ch.send("Czar " + arg + " został skasowany.")
    else:
        ch.send("Nie ma takiego czaru.")

def cmd_slist(ch, cmd, arg):
    '''Usage: slist

    Show list of spells.'''
    ch.page('\r\n'.join(spell_names))

def cmd_srename(ch, cmd, arg):
    '''Usage: srename \'<nazwa czaru>\' \'<nowa nazwa>\'

    Rename selected spell.'''
    if len(arg) == 0:
        ch.send('Prawidłowa forma to: srename \'<nazwa czaru>\' \'<nowa nazwa>\'')
        return
    if arg.count("'") != 4:
        ch.send('Musisz podać starą i nową nazwę czaru.')
        return
    index = arg.index("'", 1)
    oldname = arg[1:index]
    index = arg.index("'", index + 1) + 1
    newname = arg[index:-1]
    index = spell_names.index(oldname)
    spell_names[index] = newname
    __spells__[index].name = newname
    save_spells()
    ch.send("Nazwa czaru " + oldname + " zmieniona na " + newname + ".")

def cmd_cast(ch, cmd, arg):
    '''Usage: rzuc <spell name> [target]

    Cast selected spell on selected target. During battle, you can cast spell
    on your current target without specify it. Area spells don\'t need target too.'''
    #if we are in anti-magic room, no point in continue
    if ch.room.hasBit('antimagic'):
        ch.send('Nie możesz tutaj rzucać czarów.')
        return
    if len(arg) == 0:
        if not ch.hasvar('spell'):
            ch.send('Prawidłowa składnia to: rzuc <nazwa czaru> [cel]')
            return
        else:
            arg = ch.getvar('spell')
    parts = arg.split()
    if len(parts) > 2:
        spellname = ' '.join(parts[0:-1])
    else:
        spellname = parts[0]
    spellname = spellname.capitalize()
    target = parts[-1]
    if target == spellname:
        if ch.hasvar('enemy'):
            target = ch.getvar('enemy')
    #did char know this spell?
    if spellname not in ch.aux('spells').names:
        spellname = arg
        if spellname not in ch.aux('spells').names:
            ch.send('Nie znasz takiego czaru.')
            return
    #did char have enough energy to cast it?
    cindex = ch.aux('spells').names.index(spellname)
    sindex = spell_names.index(spellname)
    cost = max(__spells__[sindex].cost * (1 - (ch.aux('spells').levels[cindex] / 100)), 1)
    if ch.aux('stats').values[5] + cost > ch.aux('stats').secondvalues[5] and ch.is_pc:
        ch.send('Nie masz sił aby rzucić ten czar.')
        return
    #now, try find a target
    if __spells__[sindex].target == 'char':
        if target != spellname:
            multi = False
            try:
                tgt, = mud.parse_args(ch, True, cmd, target, 'ch.room.noself')
            except:
                return
        else:
            tgt = ch.room.chars
            tgt.remove(ch)
            multi = True
    elif __spells__[sindex].target == 'object':
        if target != spellname:
            multi = False
            try:
                tgt, = mud.parse_args(ch, True, cmd, target, 'obj.room.inv.eq')
            except:
                return
        else:
            tgt = ch.room.contents
            multi = True
    elif __spells__[sidex].target == 'self':
        tgt = ch
        multi = False
    #if this is battle spell, prevent cast it if player is not in fight or
    #fight is not allowed
    if __spells__[sindex].group == 'battle':
        if not multi:
            victs = [tgt]
        else:
            if __spells__[sindex].area:
                victs = tgt
            else:
                victs = [tgt[0]]
        targets = []
        for vict in victs:
            if vict.is_pc and not ch.room.hasBit('pkarena') and ch.is_pc:
                continue
            if len(vict.aux('fight').enemy) > 0:
                inparty = False
                try:
                    other, = mud.parse_args(ch, False, cmd, vict.aux('fight').enemy, 'ch.room')
                    if ch.hasvar('partyleader') and other.hasvar('partyleader'):
                        if ch.getvar('partyleader') == other.getvar('partyleader'):
                            inparty = True
                except:
                    continue
                if not inparty:
                    continue
            if vict.hasPrefs('immortal'):
                continue
            if vict.hasvar('partyleader') and ch.hasvar('partyleader'):
                if vict.getvar('partyleader') == ch.getvar('partyleader'):
                    continue
            targets.append(vict)
        if len(targets) == 0:
            ch.send('Nie ma celów dla tego czaru.')
            return
        tgt = targets
        multi = True
    else:
        if len(ch.aux('fight').enemy) > 0:
            if ch.is_pc:
                ch.send('Nie możesz rzucić tego czaru będąc w walce.')
            return
    #did we cast a spell?
    rand = random.randint(0, 100)
    chance = ch.aux('stats').values[2] + ch.aux('spells').levels[cindex] - __spells__[sindex].difficult + rand
    if chance < 100:
        ch.aux('spells').exp[cindex] += 1
        gainlevel.check_exp(ch, spellname, 'spell')
        ch.aux('stats').secondvalues[2] += 1
        gainlevel.check_exp(ch, 'int', 'stat')
        ch.send('Nie udało ci się rzucić czaru.')
        return
    ch.aux('spells').exp[cindex] += __spells__[sindex].difficult
    gainlevel.check_exp(ch, spellname, 'spell')
    ch.aux('stats').secondvalues[2] += __spells__[sindex].difficult
    gainlevel.check_exp(ch, 'int', 'stat')
    if ch.is_pc:
        ch.aux('stats').values[5] += cost
    ch.setvar('spell', spellname)
    ch.send('Rzuciłeś czar ' + spellname + '.')
    #check if our targets are antimagic
    targets = []
    if not multi:
        victs = [tgt]
    else:
        if __spells__[sindex].area:
            victs = tgt
        else:
            victs = [tgt[0]]
    for vict in victs:
        if __spells__[sindex].target in ['char', 'self']:
            if vict.hasPrefs('antimagic'):
                continue
        elif __spells__[sindex].target == 'object':
            if utils.is_keyword(obj.bits, 'antimagic'):
                continue
        targets.append(vict)
    if len(targets) == 0:
        ch.send('Ale nie wywołał on jakiegokolwiek efektu.')
        return
    tgt = targets
    for vict in tgt:
        if vict.is_pc:
            vict.send(ch.name + ' rzucił na ciebie jakiś czar.')
    #if spell have duration, start count it
    if __spells__[sindex].duration > 0:
        for vict in tgt:
            vict.aux('cooldowns').names.append('+Czar ' + spellname + ' przestaje działać. ' + ch.name)
            vict.aux('cooldowns').times.append(__spells__[sindex].duration)
    #now we check what spell do
    spelltype = __spells__[sindex].type.split('.')
    value = __spells__[sindex].value
    #damage spells
    if spelltype[0] == 'damage':
        for vict in tgt:
            ch.aux('fight').oldAttackType = ''
            fight.do_fight(ch, vict, 'spell')
        #if any victim survive and is npc, start normal fight
        for vict in tgt:
            if vict.is_npc:
                try:
                    vict2, = mud.parse_args(ch, False, cmd, vict.name, 'ch.room.noself')
                except:
                    continue
                vict2.act('zabij ' + ch.name)
                if not multi:
                    break
    #stats bonuses spells
    elif spelltype[0] == 'stat':
        for vict in tgt:
            if spelltype[1] == 'values':
                vict.aux('stats').values[spelltype[2]] += value
                afftype = 'cecha '
                if spelltype[2] == '0':
                    ch.aux('stats').secondvalues[4] += value
                    ch.aux('stats').secondvalues[5] += value
                elif spelltype[2] == '3':
                    ch.aux('stats').secondvalues[4] += value
                    ch.aux('stats').secondvalues[5] += value
                    ch.aux('stats').secondvalues[6] += value
                    ch.aux('stats').secondvalues[7] += value
            else:
                vict.aux('stats').secondvalues[spelltype[2]] += value
                afftype = 'odporność na '
            if value > 0:
                affprefix = 'Zwiększona '
            else:
                affprefix = 'Zmniejszona '
            vict.aux('affects').names.append(affprefix + afftype + vict.aux('stats').names[number])
            vict.aux('affects').types.append('czasowe')
            vict.aux('affects').sources.append(spellname + ';' + ch.name)
    #skills bonuses spells
    elif spelltype[0] == 'skills':
        for vict in tgt:
            if spelltype[1] not in vict.aux('skills').names:
                vict.addskill(spelltype[1], 0, value)
                number = vict.aux('skills').names.index(spelltype[1])
            else:
                number = vict.aux('skills').names.index(spelltype[1])
                vict.aux('skills').values[number] += value
            if value > 0:
                affprefix = 'Zwiększona umiejętność '
            else:
                affprefix = 'Zmniejszona umiejętność '
            vict.aux('affects').names.append(affprefix + vict.aux('skills').names[number])
            vict.aux('affects').types.append('czasowe')
            vict.aux('affects').sources.append(spellname + ';' + ch.name)

################################################################################
# initialization
################################################################################
load_spells()

auxiliary.install("spells", SkillsData, "character")

#add commands
mudsys.add_cmd('sedit', None, cmd_sedit, 'admin', False)
mudsys.add_cmd('sdelete', None, cmd_sdelete, 'admin', False)
mudsys.add_cmd('slist', None, cmd_slist, 'admin', False)
mudsys.add_cmd('srename', None, cmd_srename, 'admin', False)
mudsys.add_cmd('rzuc', None, cmd_cast, 'player', False)

olc.extend("medit", "M", spellsolc_extend_menu, spellsolc_choice_exec, None, None,
           spellsolc_to_proto)
olc.extend("pcedit", "M", spellsolc_extend_menu, spellsolc_choice_exec, None, None, None)

mudsys.create_bit("room_bits", "antimagic")
mudsys.create_bit('char_prfs', 'antimagic')
mudsys.create_bit('obj_bits', 'antimagic')

mudsys.add_char_method('addspell', add_spell)

#hook for spells
hooks.add('cooldown', spell_fading_hook)

def __unload__():
    '''remove all hooks.'''
    hooks.remove('cooldown', spell_fading_hook)
