# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010,2011  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
teams.py

Provide data and commands for player teams.
'''
import mudsys, hooks, mud

################################################################################
# functions
################################################################################
def party_invite(leader, ch):
    '''Set party invitations.'''
    invite = leader.getvar('partyinvitations').split(';')
    invite.remove(ch.name)
    leader.setvar('partyinvitations', ';'.join(invite))
    ch.delvar('partyinvite')

def left_party(ch):
    '''Left party by character.'''
    if not ch.hasvar('partyleader'):
        ch.send('Nie jesteś w drużynie.')
        return
    lname = ch.getvar('partyleader')
    ch.delvar('partyleader')
    if lname != ch.name:
        leader, = mud.parse_args(ch, False, '', lname, 'ch.world')
        members = leader.getvar('partymembers').split(';')
        members.remove(ch.name)
        if len(members) > 0:
            leader.setvar('partymembers', ';'.join(members))
        else:
            leader.delvar('partymembers')
            leader.delvar('partyleader')
            leader.delvar('partyinvitations')
        message = ch.name + ' opuszcza waszą drużynę.'
        ch.send('Opuszczasz drużynę.')
        leader.send(message)
    else:
        members = ch.getvar('partymembers').split(';')
        for member in members:
            vict, = mud.parse_args(ch, False, '', member, 'ch.world')
            vict.delvar('partyleader')
        ch.delvar('partymembers')
        ch.delvar('partyleader')
        ch.delvar('partyinvitations')
        message = ch.name + ' rozwiązuje waszą drużynę.'
        ch.send('Rozwiązujesz drużynę.')
    for member in members:
        vict, = mud.parse_args(ch, False, '', member, 'ch.world')
        vict.send(message)

def set_party(self, leader):
    '''Set party for character (only NPC).'''
    if self.is_pc:
        return
    if leader == 'me':
        self.setvar('partyleader', self.name)
        self.setvar('partymembers', self.name)
    else:
        for ch in self.room.chars:
            if ch.name == leader and ch.hasvar('partyleader') and ch.hasvar('partymembers') and ch.is_npc:
                self.setvar('partyleader', ch.name)
                self.act('podazaj ' + ch.name)
                members = ch.getvar('partymembers').split(';')
                members.append(self.name)
                ch.setvar('partymembers', ';'.join(members))
                break

def break_party(ch):
    '''Break NPC party.'''
    if not ch.hasvar('partyleader') or ch.is_pc:
        return
    #is party leader
    if ch.hasvar('partymembers'):
        members = ch.getvar('partymembers').split(';')
        ch.delvar('partymembers')
        ch.delvar('partyleader')
        members.remove(ch.name)
        if len(members) == 0:
            return
        for mob in ch.room.chars:
            if mob.name == members[0] and mob.hasvar('partyleader') and mob.is_npc:
                if mob.getvar('partyleader') == ch.name:
                    if len(members) > 1:
                        mob.setvar('partyleader', mob.name)
                        mob.setvar('partymembers', members)
                    else:
                        mob.delvar('partyleader')
                    break
    else:
        for mob in ch.room.chars:
            if mob.name == ch.getvar('partyleader') and mob.hasvar('partymembers') and mob.is_npc:
                members = ch.getvar('partymembers').split(';')
                members.remove(ch.name)
                if len(members) == 0:
                    mob.delvar('partyleader')
                    mob.delvar('partymembers')
                else:
                    mob.setvar('partymembers', ';'.join(members))
                break

################################################################################
# game commands
################################################################################
def cmd_team(ch, cmd, arg):
    '''Usage: druzyna <zapros | wyrzuc | odrzuc | odejdz | przyjmij | lista | cofnij> [character]

    Invite or disband part member. For party members - left or join party.'''
    if len(arg) == 0:
        ch.send('Prawidłowa składnia to: druzyna <zapros | wyrzuc | odrzuc | odejdz | przyjmij | lista | cofnij> [gracz]')
        return
    options = arg.split()
    if not options[0] in ['zapros', 'wyrzuc', 'odrzuc', 'odejdz', 'przyjmij', 'lista', 'cofnij']:
        ch.send('Dostępne opcje to: zapros, wyrzuc, odrzuc, odejdz, przyjmij, lista bądź cofnij.')
        return
    #list members of party
    if options[0] == 'lista':
        if not ch.hasvar('partyleader'):
            ch.send('Nie jesteś w drużynie.')
            return
        lname = ch.getvar('partyleader')
        if lname != ch.name:
            leader, = mud.parse_args(ch, False, cmd, lname, 'ch.world')
        else:
            leader = ch
        members = leader.getvar('partymembers').split(';')
        buf = ['Lista członków twojej drużyny:', lname + ' (przywódca)']
        for member in members:
            buf.append(member)
        invitations = leader.getvar('partyinvitations').split(';')
        for invitation in invitations:
            buf.append(invitation + ' (zaproszony)')
        ch.send('\r\n'.join(buf))
    #left party
    if options[0] == 'odejdz':
        left_party(ch)
    #decline party invitation
    elif options[0] == 'odrzuc':
        if not ch.hasvar('partyinvite'):
            ch.send('Nie masz zaproszenia do jakiejkolwiek drużyny.')
            return
        lname = ch.getvar('partyinvite')
        leader, = mud.parse_args(ch, False, cmd, lname, 'ch.world')
        party_invite(leader, ch)
        leader.send(ch.name + ' odrzucił twoje zaproszenie do drużyny.')
        ch.send('Odrzuciłeś zaproszenie ' + lname + '$a do drużyny.')
    #accept party invitation
    elif options[0] == 'przyjmij':
        if not ch.hasvar('partyinvite'):
            ch.send('Nie masz zaproszenia do jakiejkolwiek drużyny.')
            return
        lname = ch.getvar('partyinvite')
        leader, = mud.parse_args(ch, False, cmd, lname, 'ch.world')
        party_invite(leader, ch)
        ch.setvar('partyleader', lname)
        if leader.hasvar('partyleader'):
            members = leader.getvar('partymembers').split(';')
            members.append(ch.name)
            leader.setvar('partymembers', ';'.join(members))
            for member in members:
                vict, = mud.parse_args(ch, False, cmd, member, 'ch.world')
                vict.send(ch.name + ' dołączył do waszej drużyny.')
        else:
            leader.setvar('partyleader', lname)
            leader.setvar('partymembers', ch.name)
            leader.send(ch.name + ' dołączył do twojej drużyny.')
        ch.send('Dołączyłeś do drużyny ' + lname + '$a.')
    #drop member from party
    elif options[0] == 'wyrzuc':
        if not ch.hasvar('partymembers'):
            ch.send('Tylko przywódca drużyny może wyrzucać jej członków.')
            return
        if len(options) < 2:
            ch.send('Kogo chcesz wyrzucić z drużyny?')
            return
        if options[1] == ch.name:
            ch.send('Nie możesz wyrzucić sam siebie.')
            return
        members = ch.getvar('partymembers').split(';')
        if not options[1] in members:
            ch.send('Ten gracz nie należy do twojej drużyny.')
            return
        member, = mud.parse_args(ch, False, cmd, member, 'ch.world')
        member.delvar('partyleader')
        members.remove(member.name)
        member.send('Zostałeś wyrzucony z drużyny.')
        if len(members) > 0:
            ch.setvar('partymembers', ';'.join(members))
            for member in members:
                vict, = mud.parse_args(ch, False, cmd, member, 'ch.world')
                vict.send(member + ' został wyrzucony z waszej drużyny.')
        else:
            ch.delvar('partymembers')
            ch.delvar('partyleader')
            ch.delvar('partyinvitations')
            ch.send('Rozwiązałeś drużynę.')
    #invite player to party
    elif options[0] == 'zapros':
        if not ch.hasvar('partymembers') and ch.hasvar('partyleader'):
            ch.send('Tylko przywódca drużyny może zapraszać do niej członków.')
            return
        if len(options) < 2:
            ch.send('Kogo chcesz zaprosić do drużyny?')
            return
        if options[1] == ch.name:
            ch.send('Nie możesz zaprosić sam siebie.')
            return
        if ch.hasvar('partymembers'):
            members = ch.getvar('partymembers').split(';')
            if options[1] in members:
                ch.send('Ten gracz należy już do twojej drużyny.')
                return
            amount = len(members)
            if amount == 5:
                ch.send('Nie możesz zaprosić więcej osób do swojej drużyny.')
                return
        else:
            amount = 0
        if ch.hasvar('partyinvitations'):
            invitations = ch.getvar('partyinvitations').split(';')
            if len(invitations) + amount == 5:
                ch.send('Nie możesz zaprosić więcej osób do swojej drużyny.')
                return
        else:
            invitations = []
        try:
            tgt, = mud.parse_args(ch, True, cmd, options[1], 'ch.world')
        except:
            return
        if tgt.is_npc:
            ch.send("Nie możesz zapraszać NPC'ków do swojej drużyny.")
            return
        if tgt.hasvar('partyinvite'):
            ch.send('Ten gracz ma już zaproszenie do drużyny.')
            return
        if tgt.hasvar('partyleader'):
            ch.send('Ten gracz już należy do jakiejś drużyny.')
            return
        invitations.append(tgt.name)
        ch.setvar('partyinvitations', ';'.join(invitations))
        tgt.setvar('partyinvite', ch.name)
        tgt.send(ch.name + ' wysłał tobie zaproszenie do drużyny.')
        ch.send('Wysłałeś zaproszenie do ' + tgt.name + '$a.')
    #remove invitiation to party
    elif options[0] == 'cofnij':
        if not ch.hasvar('partymembers') and ch.hasvar('partyleader'):
            ch.send('Tylko przywódca drużyny może cofać zaproszenia do niej.')
            return
        if len(options) < 2:
            ch.send('Kogo chcesz zaprosić do drużyny?')
            return
        try:
            tgt, = mud.parse_args(ch, True, cmd, options[1], 'ch.world')
        except:
            return
        if not tgt.hasvar('partyinvite'):
            ch.send('Ten gracz nie ma zaproszenia do jakiejkolwiek drużyny.')
            return
        if tgt.getvar('partyinvite') != ch.name:
            ch.send('Nie wysłałeś zaproszenia temu graczowi.')
            return
        if not ch.hasvar('partyinvitations'):
            ch.send('Nie wysyłałeś jeszcze jakikolwiek zaproszeń do drużyny.')
            return
        party_invite(ch, tgt)
        ch.send('Cofnąłeś zaproszenie do drużyna dla ' + tgt.name + '$a.')
        tgt.send(ch.name + ' cofnął swoje zaproszenie do drużyny.')

################################################################################
# team hooks
################################################################################
def clear_team_hook(info):
    '''When player left game, he left team too.'''
    ch, = hooks.parse_info(info)
    if ch.is_pc:
        left_party(ch)
    else:
        break_party(ch)

def clear2_team_hook(info):
    '''When player enter game, be sure, that he isn\'t in party.'''
    ch, = hooks.parse_info(info)
    if ch.is_pc:
        ch.delvar('partyleader')
        ch.delvar('partyinvite')
        ch.delvar('partyinvitations')
        ch.delvar('partymembers')

################################################################################
# initialization
################################################################################
mudsys.add_cmd("druzyna", None, cmd_team, "player", False)

hooks.add('char_from_game', clear_team_hook)
hooks.add('char_to_game', clear2_team_hook)

mudsys.add_char_method('setparty', set_party)

def __unload__():
    '''removes the hooks for teams.'''
    hooks.remove('char_from_game', clear_team_hook)
    hooks.remove('char_to_game', clear2_team_hook)
