# -*- coding: utf-8 -*-
###########################################################################
#
# Copyright (C) 2010  Bartek thindil Jasicki
#
# This file is part of Anaria.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
'''
titles.py

Provide data and functions for players titles.
'''

import mudsys, mud, auxiliary, storage, olc, mudsock

################################################################################
# titles type definition
################################################################################

class TitleData:
    '''Data for titles.'''

    def __init__(self, set = None):
        '''Standard class constructor.'''
        self.title = ""
        self.pretitle = ""

        if set != None:
            self.title = set.readString("title")
            self.pretitle = set.readString("pretitle")

    def copyTo(self, to):
        to.title = self.title
        to.pretitle = self.pretitle

    def copy(self):
        newData = TitleData()
        self.copyTo(newData)
        return newData

    def store(self):
        set = storage.StorageSet()
        set.storeString("title", self.title)
        set.storeString("pretitle", self.title)
        return set

################################################################################
# Title OLC
################################################################################

def titleolc_menu(sock, data):
    '''Show title menu.'''
    buf = []
    if len(data.aux('title').title) == 0:
        title = "brak"
    else:
        title = data.aux('title').title
    if len(data.aux('title').pretitle) == 0:
        pretitle = "brak"
    else:
        pretitle = data.aux('title').pretitle
    buf.append("{g1) Tytuł: {c%s{g" % title)
    buf.append("{g2) Przedrostek: {c%s{g" % pretitle)
    sock.send("\r\n".join(buf))

def titleolc_chooser(sock, data, option):
    '''Try pick title to set.'''
    if option == '1':
        sock.send("Nowy tytuł: ")
        return 1
    elif option == '2':
        sock.send("Nowy przedrostek: ")
        return 2
    else:
        return olc.MENU_CHOICE_INVALID

def titleolc_parser(sock, data, choice, arg):
    '''Set new values for titles.'''
    try:
        option = int(choice)
    except:
        return False
    if option == 1:
        data.aux('title').title = arg
        if len(arg) > 0:
            arg = ' ' + arg
        if len(data.aux('title').pretitle) > 0:
            data.rdesc = data.aux('title').pretitle + ' ' + data.name + arg + ' jest tutaj.'
        else:
            data.rdesc = data.name + arg + ' jest tutaj.'
        return True
    elif option == 2:
        data.aux('title').pretitle = arg
        if len(arg) > 0:
            arg = arg + ' '
        if len(data.aux('title').title) > 0:
            data.rdesc = arg + data.name + ' ' + data.aux('title').title + ' jest tutaj.'
        else:
            data.rdesc = arg + data.name + ' jest tutaj.'
        return True
    else:
        return False

def titleolc_to_proto(data):
    '''Save titles.'''
    buf = 'me.aux("title").title = %s\n' % data.aux('title').title
    buf += 'me.aux("title").pretitle = %s\n' % data.aux('title').pretitle
    return buf

def titleolc_extend_menu(sock, data):
    '''Show option in main menu.'''
    sock.send('Edytuj tytuł i przedrostek')

def titleolc_choice_exec(sock, data):
    '''Edit title.'''
    olc.do_olc(sock, titleolc_menu, titleolc_chooser, titleolc_parser, None, data)
    return olc.MENU_NOCHOICE

################################################################################
# game commands
################################################################################
def cmd_title(ch, cmd, arg):
    '''Set player new title.'''
    #if no argument, send information about title to player
    if len(arg) == 0:
        info = ''
        if len(ch.aux('title').title) == 0:
            info = 'Nie masz obecnie ustawionego jakiegokolwiek tytułu.'
        else:
            info = 'Twój obecny tytuł to: ' + ch.aux('title').title + '.'
        ch.send(info)
        return
    #limit title to max 15 characters
    if len(arg) > 15:
        ch.send("Tytuł może miec maksymalnie 15 znaków długości. Ten jest za długi.")
        return
    #clean player title
    if arg == "czysc":
        arg = ''
    ch.aux('title').title = arg
    if len(arg) > 0:
        arg = ' ' + arg
    if len(ch.aux('title').pretitle) > 0:
        ch.rdesc = ch.aux('title').pretitle + ' ' + ch.name + arg + ' jest tutaj.'
    else:
        ch.rdesc = ch.name + arg + ' jest tutaj.'
    if len(arg) > 0:
        ch.send("Ustawiłeś swój tytuł na:" + arg)
    else:
        ch.send("Skasowałeś swój tytuł.")

################################################################################
# initialization
################################################################################
auxiliary.install("title", TitleData, "character")

olc.extend("pcedit", "T", titleolc_extend_menu, titleolc_choice_exec, None, None, None)

mudsys.add_cmd("tytul", None, cmd_title, "player", False)
